# cd_hit_clstr_file = '/lustre/home/xuyuxing/Database/Cuscuta/Cau/genomev1.1/centromere/Cau_patten.fasta.cd-hit.clstr'
from config import cd_hit_dir_path
from lib.common.os import mkdir, rmdir, cmd_run
import uuid
import os
import sys
import re
import string
from lib.common.genome.seq_base import read_fasta_big
from lib.xuyuxing.base import base_function as bf


def psi_cd_hit(input_file, output_prefix, wd, c=0.8):
    psi_cd_hit_path = cd_hit_dir_path + "/psi-cd-hit/psi-cd-hit.pl"
    cmd_string = '%s -i %s -o %s -c %.1f -G 1 -g 1 -prog blastn -exec local -para 8 -blp 4' % (
        psi_cd_hit_path, input_file, output_prefix, c)
    print(cmd_string)
    cmd_run(cmd_string, cwd=wd, silence=True)


def read_cd_hit_cluster(cd_hit_clstr_file):
    cluster_dir = {}
    with open(cd_hit_clstr_file, 'r') as f:
        all_text = f.read()
        all_text = re.sub('\r\n', '\n', all_text)
        # info = string.split(all_text, '>') python2
        info = all_text.split('\n>')
        while '' in info:
            info.remove('')
        for i in info:
            # seq = string.split(i, '\n', 1) python2
            seq = i.split('\n', 1)
            cluster_name = re.sub('>', '', seq[0])
            cluster_dir[cluster_name] = []
            cluster_info_detail = seq[1].split('\n')
            while '' in cluster_info_detail:
                cluster_info_detail.remove('')
            for info_clster in cluster_info_detail:
                ID, length, seq_name, detail = re.search(
                    r'^(\d+)\t(\d+)aa, >(\S+)\.\.\.(.*)$', info_clster).groups()
                cluster_dir[cluster_name].append(seq_name)
    return cluster_dir


def cd_hit_runing(seq_dict, args_string='-c 0.9 -n 5 -M 800 -d 0 -T 1', tmp_dir='/tmp'):
    """
    seq_dict = {
        'seq_name1': 'AA_SEQ'
    }

    seq_dict = {"T13333N0C0001G00020_0": "RYHHPSRRCPAAVAAAASHRRRPAAAASRRHRPAVAASSRRRCQPPLAAAAPASRSCCRCPCRR", 
                "T13333N0C0001G00117_0": "FDEMPKKDVVLYNCVIDGHAKKGSLDRCLVLLQQMKGEGTRPNAATFVGLLALCASSGSLGIGRRIHELVENEKLEMEVSLGTALVDMYAKNGCLEEAVLVFRKVRNPDVKAWTAMIMGFAVHGRSEDALRLSDEMEESGLRPNEVTLLAVLSACCHGGLVNKGKEYFKRIIRRRISLDLITFNTVINGFCKFGKLRIAVDLLTEMRSWGVEPSVVTYNTLIGGFCKSNQGGKLYRAEALLRKMVSAGIYPNAVTFNTLIDGFCKDSNSSAGFKFLEEMKRQEVSPNTVSYNTLINGLCNEGKMEEAVALLRQMESKDVIPNVVTFNTFINGFCKKGMLGDAKRFLDDIFDRGLISSTITYNTLIDGYCRLGQMMDALELKGLMLEKGISLDVSTYNCLILGFCEKGDMKAASCLLDEMAKKGIQADVITYNILISALCGNGEFKKASKLLDEMFDVGLKPSHLTYNTLMEGFCKKGNLTAALNIKARMEGNKKHANVATYNILVKCLCYKG", 
                "T13333N0C0001G00117_1": "FDEMPKKDVVLYNCVIDGHAKKGSLDRCLVLLQQMKGEGTRPNAATFVGLLALCASSGSLGIGRRIHELVENEKLEMEVSLGTALVDMYAKNGCLEEAVLVFRKVRNPDVKAWTAMIMGFAVHGRSEDALRLSDEMEESGLRPNEVTLLAVLSACCHGGLVNKGKEYFKRIIRRRISLDLITFNTVINGFCKFGKLRIAVDLLTEMRSWGVEPSVVTYNTLIGGFCKSNQGGKLYRAEALLRKMVSAGIYPNAVTFNTLIDGFCKDSNSSAGFKFLEEMKRQEVSPNTVSYNTLINGLCNEGKMEEAVALLRQMESKDVIPNVVTFNTFINGFCKKGMLGDAKRFLDDIFDRGLISSTITYNTLIDGYCRLGQMMDALELKGLMLEKGISLDVSTYNCLILGFCEKGDMKAASCLLDEMAKKGIQADVITYNILISALCGNGEFKKASKLLDEMFDVGLKPSHLTYNTLMEGFCKKGNLTAALNIKARMEGNKKHANVATYNILVKCLCYKG", 
                "T13333N0C0001G00117_2": "LHSMLKIDCVLNAHVYTPILQAYCAQGKLHLAAQLLERMRAFGCPPDVVSYNVLIHALCKNGNFQDVDRILAESSENGWEPDSVSYNTYISGLCSMGMTDKALKQLDVMLSKALHPTDVTLNILLGLLCRRSNFVEIVFFLEKSLELGLPVGAATYNTAMTRLCEIGEFSAVLNLLSDMFKKRIQPDVQTFNIVILSFCRAGRIDKAKSFFKSRGFSPDIVTCNILLHEFCLAGEFDEFYKMLAEMSSGQVSPDAITYNIIVDGLCWDGKYSKAIEFVKSINFEFKSDHTPDPLCYSAAMDTLIRAGRCREAEAVFSRMLSSGISPDSVTYSILVNMHCFHRKHFASALHLIELMRDRGCEPDVVTYSTVIAGLSRSGKLDEAFSVLHSMLKTDCVLNAHAYTPILQVYCAQGKLHLAAQLLERMRAFGCPPDVVSYNVLIRALCKNGNFQAVDRILAESSENGWEPDSISYNTYISGLCNMGMTDKALKQLDVMLSKALHPTDVTLNILLGLLCRR", 
                "T13333N0C0001G00117_3": "MEGMIKKGREPDVVSYTTIISGLCDNRRFTAAVEIWDEMWKRGIKPDVITCGSLIFGLCHGKVDVAYEIMMEAIKNNCDLSVPIYNAVINGFCRAGRVDKAKSIMALMRRNGLKLDLTTCNILLNYWCKAGMLDQAEKVIQYMKHSGINPDSYSYNQLLKGLCDTKRIEKAYKLFVNTIETEGMCDIISCNILIDGLCRARRTSSAHKLFRELDLKGIKPDTVTYGTLINGFLIAGNDKFAKELFDEMFKAGVVPNANAYSIMIHYFCKSGQIELACSYFYDMIRGGSSPDAVSYSIIINGLLRAHRVNEAMAFYEDMWNRGVIPDDGTYKLLIGGLLMENKVLLAHMIWDNMMEKGVTLDKTVSERLVEAL", 
                "T13333N0C0001G00117_4": "PSVFTYNIMIDSLCKDGDVRSAEELFCKMKQRSCSPDTVTFNSLIDGHGKCGELEEAASLLKEMMFHGCIPDLITYNALINCFCKFGRLHGAFGYFNEMRRNGLKPNVVTYSTFVDALCKKDMMQMATKFFIDMRMRGLRPNEFTFACLIDGNCKAGNLQEALLLLEEMLQDGIELNVVTYSILVDGLCKNGFLREAEEVLQTLVKSGMPMNQIPYTALIHAHFKNKNIKRAVNLMNEMRENGHKPDLSAYGTAIWGLSNEGKFDDAEVLLEEIDKCGLKPNCVIYTALLVANFKSGNTSMALNFFHKMQDLGISPTVVTYGALVDGLCKQGSVHEAFYHFNKLRDLGLQPNVLTYTSLVDRLCKDGCLEEASKLFEEMVGNGISPDKVAYTSLMHCYMKRGNLQEAFVLRDKMIENGFKLDLCAYNTIIWGLC", 
                "T13333N0C0001G00117_5": "FQQVARRLIPPDVNTYNTLLHGLIKEKNMERAVELWKHMDQGFFQPNLVSYTILIDGYCKIGRVEEASGLLNEMNNKGVFPDVVVYSALINGFCNVGTVDRGLQVFKEMGEMGILPNVVTFSCLINALGKAGRWVVVRELLNKMVDLGIQPDVFAYTGLVDGLCKEGKEIEAIKMLGLMTERGVEPSTVTYNVLINHLCKKGMIDDAIAFMKKMSGKGKVPDVVTFNTLILGLCKANRMEVAIKQMNAMLEGKHGVEPDVVTFNTIMHWLCKRGEVSQALELRKMMEERGCDFNLMTYNILIGGLIKVHKLNEAMKLQKEMADKGLRPNSFTYSTMINELSKLGKFDDAERFLHEMEDRGLSPSLMDYNTVTYTILINAVCRSRNLREATRLLGVLKHSGIMPDCYLYNCIMKAHCLLDQCWEVIEVYNRMRDEGIEPDLVTYNTLIYGLSKAGMIDQAKKLLNVMADTGYFPDVASYTSLIGGMCRKGDASGALLLLEEMEGRGCSPNECTYNALLMGLCKAGSLDKGMQ",
                "T13333N0C0001G00117_6": "LHEISEQGLHPSATTYSMLMHSLCKHGRLKEAFEVYETMEREGCHPDSVTFNILISGLCEDGKVDDGMGLLKTMQLKGCCPDAGTYQALLYGMLKDKRFVDAKNFMCKMISERRYPSFLSYKMTIEGLSMDALVRAGRCREAEAVFSRMLSSGILPDSVTYSILVKMHCFHRKHFASALHLIELMRDRGCQPDVVTYSTVIAGLSRSGKLDEAFAVLHSMLKIDCVLNAHVYTPILQAYCAQGKLHLAAQLLERMRAFGCPPDVVSYNVLIHALCKNGNFQDVDRILAESSENGWEPDSVSYNTYISGLCSMGMTDKALKQLDVMLSKALHPTDVTLNILLGLLCRRSNFVEIVFFLEKSLELGLPVGAATYNTAMTRLCEIGEFSAVLNLLSDMFKKRIQPDVQTFNIVILSFCRAGRIDKAKSFFKSRGFSPDIVTCNILLHEFCLAGEFDEFYKMLAEMSSGQVSPDAITYNIIVDGLCWD", 
                "T13333N0C0001G00117_7": "RSMLNRVPKPNVVLFNTVINIYVSAGRLLEANALFDRMMQRSYYPYALTYKILISGLCKEGKFGSIAHLLIEKIKAEGYKSDIFIFNSLIHGLCKDDQLEEAFTFYGNLLLDGFVANSGTYNTLIKALLQRGKQKESLELANDMVLYDLFVDVVDLNKLIKALCKSESVDKGLDLFEKMRQKGICPSMISCNVLIVRLCRKRRVQDALELFGKM", 
                "T13333N0C0001G00117_8": "FSDMSSRNISPYIVTYNILINGYFRSGKSVGAIMTFRRLPKFGFSPEIITYNTMLHGFFKSGKLGYVSRIFKEIKESDIVPNAITYTTVMKSLLKSGKFEQGLDIFLEMISRGYASEVFAYCSIISALVKRGAIEDAYNFSRQMLNNGIGIDLACYNVLVYVYCKEKNMNKAEALQIFSDMSSRNISPSIVTYNILINGYFRSGKSVSAIMTFRRLSKFGFSPEIITYNTMLHGFLKSGKLDYVSRIFKEIKESGIVPNAITYATVMKSLLRSGKFEQGLDIFLEMMSKGYASDVFAYCFIISALVKRGATEDAYHFSRQMLNNGIGIDLACYNVLLYVYCKEKNMNKAVQLLNEIEEAGLQSDEYTYSTLVDAFCKLGDVSSAWNYLRAMEINDISMKLVAFNCLIDGLCKVGQVNHALSVFNMMKTRDSFTYSILLHGLC", 
                "T13333N0C0001G00117_9": "LFHSLRYLAIPIDLVSCNIFLNCFTQAGRLSDALHLFGEMPGRGLAPDICTFNTLIKGCCRGNQISGAFRLLDGLHVQGLTPNEYTYSMLIXGSHLAADSEIGFGLVREMVQLGLEPSGVNYGCILAAVCREAKLGEAKVVYGRMIKVGIGLDIWAYNSFIGLLGRKRMVNEAKLLFFXMPDKGVVPNVDTFNVLIHGICVNGNLEEGLRILKGMVSRGPRPNIKPYTAVLHALCRRCMFDEALELFDDFPRQGLSSDKHMFGTLLFSLCNAGRVEVASRLFYQMESSGFNDIAAYNCLICGYCKLANMVAAKKLFVEALAKGMEPDVIMYTTLIDGPFKIGDIKVAQELVRDFGESGLKPDMIMFSTLMEGLCETESLAAALELLGEIVRQGLEPNSAIYTMLICAPCMARNFGKTVHFLEEM", 
                "T13333N0C0001G00117_10": "FYQIKAYKCRPTASAYNSIIIMLMQEGHHEKVHELYDEMCNEGNCFPDSVTYSALISSFCKLGRHDSAIRLFEEMKENGMQPMPRIYTTLITLFFNSKNVDRALDLFREMKQYCCAPNVFTYTELIKGLGKVGRVEDAYHYFLEMKKEGCKPDTVLMNNLLNVLSKARRLDDVLSLFGQMESFQCIPNVVTYNTVIKTLFESKSRASKVISWFEKMKERGIAPSSFTYSILIDGFCKSNRLEEALILLEEMDEKGFPPCPAAYCSLVDSLGKAKRYEAACELFQELKENCGSFSSRVYAVMIKHLGRCGRLNDAIDLFEEMKKLGCPPDAYAYNALMSGMVRIGIFDEAWSLFRTMQEQGCKPDINSYNIILNGLAKAGGPNPAMEMLSNMKQSVCKPDAVSYNTVLGSLSRAGMFEEAAKLMKEMSAMGFEYDLITYS", 
                "T13333N0C0001G00117_11": "IRRMREAGVKPDVITYNSLMAGASTYNLSSYCRHLFEEMSKVGIRADEWSYNTLMHCLFNCGCPEEALQIFSDMSSRNISPSIVTYNILINGYFISGKSVSAIMTFRRLPEFGFSPEIITYNTMLHGFFKSGKLGYVSRIFKEIKESGIVPNAITYTTVMKSLLRSGKFEQGLDIFLEMMSRGYASDVFAYCSIISALVKRGATEDAYNFSRQMLNNGIGIDLACYNVLLYVYCKEKNMNKAVQLLNEIEEAGLQSDEYTYSTLVDGFCKLGDVKEALQIFSDMSSRNISPYIVTYNILINGYFRSGKSVGAIMTFRRLPKFGFSPEIITYNTMLHGFFKSGKLGYVSRIFKEIKESDIVPNAITYTTVMKSLLKSGKFEQGLDIFLEMISRGYASEVFAYCSIISALVKRGAIEDAYNFSRQMLNNGIGIDLACYNVLVYVYCKEKNMNKAVQLLNEI", 
                "T13333N0C0001G00117_12": "FSSLIRRYARSGRPSEAIDAFHRMSDYGLEPDPASFSILLAALSRQRMAAQAQSLFDSFSARFPPDVVTYSNLVHAWCRAGDLDKAEQAFSAMRAAGVQPNVYTYTAAIDAMCRSGQIPRAQEVLCQMLDSGCAPNAATFNCFMRAHVRAGRAEQALQIHNQMRKLGCDPDGITYNFLIEAHCSKRQRNLDAALKILNSMIAKGKGCAPDANSFNHILKCVLNLGDVSAAHKLYEKMREIGCRPNTVTYNIFLQLFSKDKTLDMVLKMKKEMEKEEVEPNVNTYGVLITTFC"
    }

    """
    # rename seq
    num_site = len(str(len(seq_dict)))
    new_id_pattern = "S%0" + str(num_site) + "d"

    num = 0
    map_dict = {}
    for i in seq_dict:
        new_id = new_id_pattern % num
        map_dict[i] = new_id
        num += 1

    map_hash_dict = {}
    for i in map_dict:
        map_hash_dict[map_dict[i]] = i

    # write tmp file    
    work_dir = os.path.abspath(tmp_dir + "/" + uuid.uuid1().hex)
    mkdir(work_dir)
    with open(work_dir+"/input.fasta", 'w') as f:
        for seq_id in seq_dict:
            f.write(">%s\n%s\n" % (map_dict[seq_id], seq_dict[seq_id]))
    
    # run
    cmd_string = '%s/cd-hit -i input.fasta -o output %s' % (cd_hit_dir_path, args_string)
    cmd_run(cmd_string, cwd=work_dir, silence=True)

    # get repsent seq
    repsent_seqname = {}
    for i in read_fasta_big(work_dir+"/output"):
        repsent_seqname[i.seqname] = 0

    # get clstr
    output_file = work_dir + "/output.clstr"

    if os.path.exists(output_file) and os.path.getsize(output_file) != 0:
        cluster_dict = read_cd_hit_cluster(output_file)
        cluster_old_name_dict = {}
        for c_id in cluster_dict:
            tmp_list = []
            for new_seq_id in cluster_dict[c_id]:
                tmp_list.append(map_hash_dict[new_seq_id])
                if new_seq_id in repsent_seqname:
                    rep_seq_id = map_hash_dict[new_seq_id]
            cluster_old_name_dict[rep_seq_id] = tmp_list
    else:
        cluster_old_name_dict = None
    
    rmdir(work_dir)

    return cluster_old_name_dict
    


# # TODO
#
#
#
# # python /share/home/xuyuxing/python_xuyuxing/Genome_work_tools/Extract_seq_from_cd-hit.py cd-hit.0.65.clstr 0.65 all.uniq.fa
#
#
# cd_hit_clstr_file = sys.argv[1]
# output_prefix = sys.argv[2]
# fasta_file = sys.argv[3]
#
# F1 = open(cd_hit_clstr_file)
# all_text = F1.read()
# info = string.split(all_text, '\n>')
# while '' in info:
#     info.remove('')
# row = []
# cluster_dict = {}
# for i in info:
#     cluster_id = re.match(r'.*(Cluster \d+)\n.*', i)
#     if cluster_id:
#         cluster_id = cluster_id.group(1)
#
#     cluster_id = re.sub(r' ', '_', cluster_id)
#
#     cluster_dict[cluster_id] = []
#
#     seqs = string.split(i, '\n')
#     for seq in seqs:
#         seq_id = re.match(r'.*>(.*)\.\.\..*', seq)
#         if seq_id:
#             seq_id = seq_id.group(1)
#             cluster_dict[cluster_id].append(seq_id)
#
# bf.mkdir(output_prefix)
#
# fasta_dict, a = seq_base.read_fasta(fasta_file)
#
# for i in cluster_dict:
#     file_path = output_prefix + "/" + i + ".seq"
#     seq_base.extract_seq_to_fasta(file_path, cluster_dict[i], fasta_dict)



import os
import re
import pickle
from pyfaidx import Fasta
from lib.common.util import printer_list
from lib.common.util import logging_init
from lib.common.os import mkdir, copy_file, cmd_run, multiprocess_running
from src.xuyuxing.tools.seqtools import split_fasta
from lib.common.genome.seq_base import read_fasta


# maker_p repeat

def step_1_3(sh_file, trna_file, crl_dir):
    with open(sh_file, 'w') as f:
        f.write("""#! /bin/bash

# http://weatherby.genetics.utah.edu/MAKER/wiki/index.php/Repeat_Library_Construction-Advanced

source activate maker_p

GENOME=$1
TAG=$2
tRNA_file=%s
CRL_DIR=%s

# 2.1.1. Collection of candidate elements with LTRs that are 99%% or more in similarity using LTRharvest
mkdir $TAG
cd $TAG

gt suffixerator -db $GENOME -indexname $TAG.index -tis -suf -lcp -des -ssp -dna
gt ltrharvest -index $TAG.index -out $TAG.out99 -outinner $TAG.outinner99 -gff3 $TAG.gff99 -minlenltr 100 -maxlenltr 6000 -mindistltr 1500 -maxdistltr 25000 -mintsd 5 -maxtsd 5 -motif tgca -similar 99 -vic 10  > $TAG.result99

# 2.1.2. Using LTRdigest to find elements with PPT (poly purine tract) or PBS (primer binding site)
gt gff3 -sort $TAG.gff99 > $TAG.gff99.sort
gt ltrdigest -trnas $tRNA_file $TAG.gff99.sort $TAG.index > $TAG.gff99.dgt

perl $CRL_DIR/CRL_Step1.pl --gff $TAG.gff99.dgt

# 2.1.3. Further filtering of the candidate elements
perl $CRL_DIR/CRL_Step2.pl --step1 CRL_Step1_Passed_Elements.txt --repeatfile $TAG.out99 --resultfile $TAG.result99 --sequencefile $GENOME --removed_repeats CRL_Step2_Passed_Elements.fasta

mkdir fasta_files
mv Repeat_*.fasta fasta_files
mv CRL_Step2_Passed_Elements.fasta fasta_files
cd fasta_files

perl $CRL_DIR/CRL_Step3.pl --directory ../fasta_files --step2 CRL_Step2_Passed_Elements.fasta --pidentity 60 --seq_c 25

mv  CRL_Step3_Passed_Elements.fasta  ..
cd  ..

        """ % (trna_file, crl_dir))


def step_1_3_85(sh_file, trna_file, crl_dir):
    with open(sh_file, 'w') as f:
        f.write("""#! /bin/bash

# http://weatherby.genetics.utah.edu/MAKER/wiki/index.php/Repeat_Library_Construction-Advanced
# 2.2. Collection of relatively old LTR retrotransposons

source activate maker_p

GENOME=$1
TAG=$2
tRNA_file=%s
CRL_DIR=%s

# 2.1.1
mkdir ${TAG}
cd ${TAG}

gt suffixerator -db $GENOME -indexname $TAG.index -tis -suf -lcp -des -ssp -dna
gt ltrharvest -index $TAG.index -out $TAG.out85 -outinner $TAG.outinner85 -gff3 $TAG.gff85 -minlenltr 100 -maxlenltr 6000 -mindistltr 1500 -maxdistltr 25000 -mintsd 5 -maxtsd 5 -vic 10  > $TAG.result85

# 2.1.2. Using LTRdigest to find elements with PPT (poly purine tract) or PBS (primer binding site)
gt gff3 -sort $TAG.gff85 > $TAG.gff85.sort
gt ltrdigest -trnas $tRNA_file $TAG.gff85.sort $TAG.index > $TAG.gff85.dgt

perl $CRL_DIR/CRL_Step1.pl --gff $TAG.gff85.dgt

# 2.1.3. Further filtering of the candidate elements
perl $CRL_DIR/CRL_Step2.pl --step1 CRL_Step1_Passed_Elements.txt --repeatfile $TAG.out85 --resultfile $TAG.result85 --sequencefile $GENOME --removed_repeats CRL_Step2_Passed_Elements.fasta

mkdir fasta_files
mv Repeat_*.fasta fasta_files
mv CRL_Step2_Passed_Elements.fasta fasta_files
cd fasta_files

perl $CRL_DIR/CRL_Step3.pl --directory ../fasta_files --step2 CRL_Step2_Passed_Elements.fasta --pidentity 60 --seq_c 25

mv  CRL_Step3_Passed_Elements.fasta  ..
cd  ..

""" % (trna_file, crl_dir))


def step3_merge(work_dir, contig_list, out_dir, identity):
    """
    work_dir = "/lustre/home/xuyuxing/Database/Gel/other/Dca/Dca.maker_p/LTR/99"
    contig_list = ['genome']
    out_dir = "/lustre/home/xuyuxing/Database/Gel/other/Dca/Dca.maker_p/LTR/99/merge"
    identity='99'
    """

    identity = str(identity)

    contig_dir = {}
    num = 0
    for i in contig_list:
        contig_dir[i] = {}
        contig_dir[i]['seq-nr'] = str(num)
        contig_main_dir = work_dir + "/" + i
        if os.path.exists(contig_main_dir):
            CRL_Step3_file = contig_main_dir + "/CRL_Step3_Passed_Elements.fasta"
            result99_file = contig_main_dir + "/" + i + ".result%s" % identity
            outinner99_file = contig_main_dir + "/" + i + ".outinner%s" % identity

            if os.path.exists(CRL_Step3_file):
                contig_dir[i]["CRL_Step3_file"] = CRL_Step3_file
            # else:
            #     print("%s miss CRL_Step3_file" % i)

            if os.path.exists(result99_file):
                contig_dir[i]["result99_file"] = result99_file
            # else:
            #     print("%s miss result99_file" % i)

            if os.path.exists(outinner99_file):
                contig_dir[i]["outinner99_file"] = outinner99_file
            # else:
            #     print("%s miss outinner99_file" % i)
        # else:
        #     print("%s miss main dir" % i)
        num = num + 1

    mkdir(out_dir, True)
    merge_CRL_Step3_file = out_dir + "/all.CRL_Step3_Passed_Elements.fasta"
    merge_result99 = out_dir + "/all.result%s" % identity
    merge_outinner99 = out_dir + "/all.outinner%s" % identity

    if len(contig_list) == 1 and contig_list[0] == 'genome':
        cmd_run("cp %s %s" % (contig_dir["genome"]["CRL_Step3_file"], merge_CRL_Step3_file))
        cmd_run("cp %s %s" % (contig_dir["genome"]["result99_file"], merge_result99))
        cmd_run("cp %s %s" % (contig_dir["genome"]["outinner99_file"], merge_outinner99))
    else:
        step3_f = open(merge_CRL_Step3_file, 'w')
        result99_f = open(merge_result99, 'w')
        outinner99_f = open(merge_outinner99, 'w')

        for contig in contig_dir:
            if "CRL_Step3_file" in contig_dir[contig]:
                CRL_Step3_file = contig_dir[contig]["CRL_Step3_file"]
                seqdict, seqname_list = read_fasta(CRL_Step3_file)
                for i in seqname_list:
                    new_seq_name = re.sub('dbseq-nr_0', 'dbseq-nr_%s' % contig_dir[contig]['seq-nr'], i)
                    step3_f.write('>%s\n%s\n' % (new_seq_name, seqdict[i].seq.lower()))

            if "result99_file" in contig_dir[contig]:
                result99_file = contig_dir[contig]["result99_file"]
                with open(result99_file, 'r') as f:
                    for each_line in f:
                        if re.match(r'^#', each_line):
                            continue
                        each_line = re.sub('\n', '', each_line)
                        info = each_line.split()
                        info[-1] = str(contig_dir[contig]['seq-nr'])
                        result99_f.write(printer_list(info, sep="  ") + "\n")

            if "outinner99_file" in contig_dir[contig]:
                outinner99_file = contig_dir[contig]["outinner99_file"]
                seqdict, seqname_list = read_fasta(outinner99_file, full_name=True, upper=False)
                for i in seqname_list:
                    new_seq_name = re.sub('dbseq-nr 0', 'dbseq-nr %s' % contig_dir[contig]['seq-nr'], i)
                    seqdict[i].wrap(60)
                    outinner99_f.write('>%s\n%s' % (new_seq_name, seqdict[i].seq.lower()))

        step3_f.close()
        result99_f.close()
        outinner99_f.close()


def step_4_5(sh_file, genome, trna_file, crl_dir, mite_file, tpase_dir):
    with open(sh_file, 'w') as f:
        f.write("""#! /bin/bash

# http://weatherby.genetics.utah.edu/MAKER/wiki/index.php/Repeat_Library_Construction-Advanced

source activate maker_p

GENOME=%s
TAG=all
tRNA_file=%s
CRL_DIR=%s
MITE_file=%s
Tpase_dir=%s

# 2.1.4. Identify elements with nested insertions

perl $CRL_DIR/ltr_library.pl --resultfile $TAG.result99 --step3 $TAG.CRL_Step3_Passed_Elements.fasta --sequencefile $GENOME
cat lLTR_Only.lib $MITE_file  > repeats_to_mask_LTR99.fasta
RepeatMasker -pa 56 -lib repeats_to_mask_LTR99.fasta -nolow -dir . $TAG.outinner99
perl $CRL_DIR/cleanRM.pl $TAG.outinner99.out $TAG.outinner99.masked > $TAG.outinner99.unmasked
perl $CRL_DIR/rmshortinner.pl $TAG.outinner99.unmasked 50 > $TAG.outinner99.clean

blastx -query $TAG.outinner99.clean -db $Tpase_dir/Tpases020812DNA -evalue 1e-10 -num_descriptions 10 -out $TAG.outinner99.clean_blastx.out.txt -num_threads 56
perl $CRL_DIR/outinner_blastx_parse.pl --blastx $TAG.outinner99.clean_blastx.out.txt --outinner $TAG.outinner99

# 2.1.5 Building examplars

perl $CRL_DIR/CRL_Step4.pl --step3 $TAG.CRL_Step3_Passed_Elements.fasta --resultfile $TAG.result99 --innerfile passed_outinner_sequence.fasta --sequencefile $GENOME

makeblastdb -in lLTRs_Seq_For_BLAST.fasta -dbtype nucl
blastn -query lLTRs_Seq_For_BLAST.fasta -db lLTRs_Seq_For_BLAST.fasta -evalue 1e-10 -num_descriptions 1000 -out lLTRs_Seq_For_BLAST.fasta.out -num_threads 56

makeblastdb -in Inner_Seq_For_BLAST.fasta -dbtype nucl
blastn -query Inner_Seq_For_BLAST.fasta -db Inner_Seq_For_BLAST.fasta  -evalue 1e-10 -num_descriptions 1000 -out  Inner_Seq_For_BLAST.fasta.out -num_threads 56

perl $CRL_DIR/CRL_Step5.pl --LTR_blast lLTRs_Seq_For_BLAST.fasta.out --inner_blast Inner_Seq_For_BLAST.fasta.out --step3 $TAG.CRL_Step3_Passed_Elements.fasta --final LTR99.lib --pcoverage 90 --pidentity 80

""" % (genome, trna_file, crl_dir, mite_file, tpase_dir))


def step_4_5_85(sh_file, genome, trna_file, crl_dir, mite_file, tpase_dir):
    with open(sh_file, 'w') as f:
        f.write("""#! /bin/bash

# http://weatherby.genetics.utah.edu/MAKER/wiki/index.php/Repeat_Library_Construction-Advanced

source activate maker_p

GENOME=%s
TAG=all
tRNA_file=%s
CRL_DIR=%s
MITE_file=%s
Tpase_dir=%s

# 2.1.4. Identify elements with nested insertions

perl $CRL_DIR/ltr_library.pl --resultfile $TAG.result85 --step3 $TAG.CRL_Step3_Passed_Elements.fasta --sequencefile $GENOME
cat lLTR_Only.lib $MITE_file  > repeats_to_mask_LTR85.fasta
RepeatMasker -pa 56 -lib repeats_to_mask_LTR85.fasta -nolow -dir . $TAG.outinner85
perl $CRL_DIR/cleanRM.pl $TAG.outinner85.out $TAG.outinner85.masked > $TAG.outinner85.unmasked
perl $CRL_DIR/rmshortinner.pl $TAG.outinner85.unmasked 50 > $TAG.outinner85.clean

blastx -query $TAG.outinner85.clean -db $Tpase_dir/Tpases020812DNA -evalue 1e-10 -num_descriptions 10 -out $TAG.outinner85.clean_blastx.out.txt -num_threads 56
perl $CRL_DIR/outinner_blastx_parse.pl --blastx $TAG.outinner85.clean_blastx.out.txt --outinner $TAG.outinner85

# 2.1.5 Building examplars

perl $CRL_DIR/CRL_Step4.pl --step3 $TAG.CRL_Step3_Passed_Elements.fasta --resultfile $TAG.result85 --innerfile passed_outinner_sequence.fasta --sequencefile $GENOME

makeblastdb -in lLTRs_Seq_For_BLAST.fasta -dbtype nucl
blastn -query lLTRs_Seq_For_BLAST.fasta -db lLTRs_Seq_For_BLAST.fasta -evalue 1e-10 -num_descriptions 1000 -out lLTRs_Seq_For_BLAST.fasta.out -num_threads 56

makeblastdb -in Inner_Seq_For_BLAST.fasta -dbtype nucl
blastn -query Inner_Seq_For_BLAST.fasta -db Inner_Seq_For_BLAST.fasta  -evalue 1e-10 -num_descriptions 1000 -out  Inner_Seq_For_BLAST.fasta.out -num_threads 56

perl $CRL_DIR/CRL_Step5.pl --LTR_blast lLTRs_Seq_For_BLAST.fasta.out --inner_blast Inner_Seq_For_BLAST.fasta.out --step3 $TAG.CRL_Step3_Passed_Elements.fasta --final LTR85.lib --pcoverage 90 --pidentity 80

""" % (genome, trna_file, crl_dir, mite_file, tpase_dir))


def maker_p_repeat_main(args):
    mkdir(args.output_dir, True)
    log_file = args.output_dir + "/log.txt"
    args.genome = os.path.abspath(args.genome)
    args.output_dir = os.path.abspath(args.output_dir)

    logger = logging_init("maker_p repeat build", log_file)

    args_info_string = "Argument list:\n"
    attrs = vars(args)

    for item in attrs.items():
        args_info_string = args_info_string + ("%s: %s\n" % item)

    logger.info(args_info_string)

    logger.info("Step1: MITE find")

    MITE_dir = args.output_dir + "/MITE"
    MITE_pass_flag_file = MITE_dir + "/MITE.ok"

    if not os.path.exists(MITE_pass_flag_file):
        logger.info("   MITE find, begin")

        bash_file = MITE_dir + "/MITE.sh"
        mkdir(MITE_dir, True)

        with open(bash_file, 'w') as f:
            f.write("""source activate maker_p && \\
cp %s %s/genome.fasta && \\
perl %s -i genome.fasta -g %s -c %d -n %d -S 12345678 && \\
cat *Step8_* > MITE.lib
""" % (args.genome, MITE_dir, args.mite_hunter_manager, args.tag, args.num_threads, args.num_threads))

        code, output, error = cmd_run("bash %s" % bash_file, cwd=MITE_dir)

        stdout_file = MITE_dir + "/MITE.sh.o"
        with open(stdout_file, 'w') as f:
            f.write(output)

        stderr_file = MITE_dir + "/MITE.sh.e"
        with open(stderr_file, 'w') as f:
            f.write(error)

        MITE_file = MITE_dir + "/MITE.lib"

        if code and os.path.getsize(MITE_file) != 0:
            MITE_pass_flag_file = MITE_dir + "/MITE.ok"
            cmd_run("touch %s" % MITE_pass_flag_file, cwd=MITE_dir)
            logger.info("   MITE find, finished")
        else:
            logger.info("   MITE find, failed, please check")
            raise ValueError("MITE failed, please check")

    else:
        logger.info("   MITE find, already finished, skip step")
        MITE_file = MITE_dir + "/MITE.lib"

    logger.info("Step2: genome split")
    if args.split_genome:
        logger.info("   begin genome split")
        split_genome_dir = args.output_dir + "/split_genome"
        mkdir(split_genome_dir, True)

        genome_ref = Fasta(args.genome)
        contig_list = list(genome_ref.keys())
        split_ok = True
        for i in contig_list:
            contig_file = split_genome_dir + "/" + i + ".fa"
            if not (os.path.exists(contig_file) and os.path.getsize(contig_file) != 0):
                split_ok = False

        if not split_ok:
            mkdir(split_genome_dir, True)
            split_fasta(args.genome, split_genome_dir, None, contig_model=True)
        logger.info("   end genome split")
    else:
        logger.info("   skip genome split")
        split_genome_dir = args.output_dir + "/split_genome"
        mkdir(split_genome_dir, True)
        copy_file(args.genome, split_genome_dir + "/genome.fa")
        contig_list = ['genome']

    logger.info("Step3: LTRs find")
    ltr_dir = args.output_dir + "/LTR"
    mkdir(ltr_dir, True)

    logger.info("Step3.1: LTRs that are 99% or more in similarity")
    ltr_99_dir = ltr_dir + "/99"
    ltr_99_log = ltr_99_dir + "/log.txt"
    mkdir(ltr_99_dir, True)
    step_1_3_sh = ltr_99_dir + "/step_1_3.sh"
    step_1_3(step_1_3_sh, args.trna_file, args.crl_dir)
    tmp_out_pickle = ltr_99_dir + "/shell_para_out.pyb"

    logger.info("   raw step 2.1.1 - 2.1.3")

    step_1_3_flag = False
    if os.path.exists(tmp_out_pickle):
        # TEMP = open(tmp_out_pickle, 'rb')
        # tmp_out = pickle.load(TEMP)
        # TEMP.close()

        crl_step_file_num = {
            'step1': [0, 0],
            'step2': [0, 0],
            'step3': [0, 0],
            'dgt': [0, 0],
            'result99': [0, 0]
        }

        for contig in contig_list:
            crl_step_file = {
                'step1': "%s/%s/CRL_Step1_Passed_Elements.txt" % (ltr_99_dir, contig),
                'step2': "%s/%s/fasta_files/CRL_Step2_Passed_Elements.fasta" % (ltr_99_dir, contig),
                'step3': "%s/%s/CRL_Step3_Passed_Elements.fasta" % (ltr_99_dir, contig),
                'dgt': "%s/%s/%s.gff99.dgt" % (ltr_99_dir, contig, contig),
                'result99': "%s/%s/%s.result99" % (ltr_99_dir, contig, contig),
            }

            for step in list(crl_step_file.keys()):
                if os.path.exists(crl_step_file[step]):
                    crl_step_file_num[step][0] += 1
                    if os.path.getsize(crl_step_file[step]) != 0:
                        crl_step_file_num[step][1] += 1

        if crl_step_file_num['step1'][0] == crl_step_file_num['step2'][0] == crl_step_file_num['dgt'][0] == \
                crl_step_file_num['result99'][1] == len(contig_list):
            if crl_step_file_num['step1'][1] != 0 and crl_step_file_num['step2'][1] != 0 and crl_step_file_num['step3'][
                1] != 0:
                step_1_3_flag = True
                logger.info("   raw step 1-3, already finished, skip step")

    if not step_1_3_flag:

        args_list = []
        for contig in contig_list:
            contig_fasta_file = os.path.abspath(split_genome_dir + "/" + contig + ".fa")
            cmd_string = "bash %s %s %s" % (step_1_3_sh, contig_fasta_file, contig)
            args_list.append((cmd_string, ltr_99_dir, 1, True, None))

        logger.info("   raw step 1-3, begin")

        tmp_out = multiprocess_running(cmd_run, args_list, args.num_threads, log_file=ltr_99_log)

        OUT = open(tmp_out_pickle, 'wb')
        pickle.dump(tmp_out, OUT)
        OUT.close()

        logger.info("   raw step 1-3, finished")

    logger.info("   merge CRL_Step3_Passed_Elements.fasta，$TAG.result99，$TAG.outinner99")
    merge_dir = ltr_99_dir + "/merge"
    mkdir(merge_dir, True)
    step3_merge(ltr_99_dir, contig_list, merge_dir, 99)

    logger.info("   raw step 2.1.4 - 2.1.5")

    LTR99_lib_file = merge_dir + "/LTR99.lib"
    if os.path.exists(LTR99_lib_file):
        logger.info("   raw step 4-5, already finished, skip step")
    else:
        logger.info("   raw step 4-5, begin")
        step_4_5_sh = merge_dir + "/step_4_5.sh"
        step_4_5(step_4_5_sh, args.genome, args.trna_file, args.crl_dir, MITE_file, args.tpase_dir)
        code, output, error = cmd_run("bash %s" % step_4_5_sh, cwd=merge_dir)

        stdout_file = merge_dir + "/step_4_5.sh.o"
        with open(stdout_file, 'w') as f:
            f.write(output)

        stderr_file = merge_dir + "/step_4_5.sh.e"
        with open(stderr_file, 'w') as f:
            f.write(error)

        logger.info("   raw step 4-5, finished")

    logger.info("Step3.2: LTRs that are 85% or more in similarity")
    ltr_85_dir = ltr_dir + "/85"
    ltr_85_log = ltr_85_dir + "/log.txt"
    mkdir(ltr_85_dir, True)
    step_1_3_85_sh = ltr_85_dir + "/step_1_3.sh"
    step_1_3_85(step_1_3_85_sh, args.trna_file, args.crl_dir)
    tmp_out_pickle = ltr_85_dir + "/shell_para_out.pyb"

    logger.info("   raw step 2.1.1 - 2.1.3")

    step_1_3_85_flag = False
    if os.path.exists(tmp_out_pickle):
        # TEMP = open(tmp_out_pickle, 'rb')
        # tmp_out = pickle.load(TEMP)
        # TEMP.close()

        crl_step_file_num = {
            'step1': [0, 0],
            'step2': [0, 0],
            'step3': [0, 0],
            'dgt': [0, 0],
            'result85': [0, 0]
        }

        for contig in contig_list:
            crl_step_file = {
                'step1': "%s/%s/CRL_Step1_Passed_Elements.txt" % (ltr_85_dir, contig),
                'step2': "%s/%s/fasta_files/CRL_Step2_Passed_Elements.fasta" % (ltr_85_dir, contig),
                'step3': "%s/%s/CRL_Step3_Passed_Elements.fasta" % (ltr_85_dir, contig),
                'dgt': "%s/%s/%s.gff85.dgt" % (ltr_85_dir, contig, contig),
                'result85': "%s/%s/%s.result85" % (ltr_85_dir, contig, contig),
            }

            for step in list(crl_step_file.keys()):
                if os.path.exists(crl_step_file[step]):
                    crl_step_file_num[step][0] += 1
                    if os.path.getsize(crl_step_file[step]) != 0:
                        crl_step_file_num[step][1] += 1

        if crl_step_file_num['step1'][0] == crl_step_file_num['step2'][0] == crl_step_file_num['dgt'][0] == \
                crl_step_file_num['result85'][1] == len(contig_list):
            if crl_step_file_num['step1'][1] != 0 and crl_step_file_num['step2'][1] != 0 and crl_step_file_num['step3'][
                1] != 0:
                step_1_3_85_flag = True
                logger.info("   raw step 1-3, already finished, skip step")

    if not step_1_3_85_flag:

        args_list = []
        for contig in contig_list:
            contig_fasta_file = os.path.abspath(split_genome_dir + "/" + contig + ".fa")
            cmd_string = "bash %s %s %s" % (step_1_3_85_sh, contig_fasta_file, contig)
            args_list.append((cmd_string, ltr_85_dir, 1, True, None))

        logger.info("   raw step 1-3, begin")

        tmp_out = multiprocess_running(cmd_run, args_list, args.num_threads, log_file=ltr_85_log)

        OUT = open(tmp_out_pickle, 'wb')
        pickle.dump(tmp_out, OUT)
        OUT.close()

        logger.info("   raw step 1-3, finished")

    logger.info("   merge CRL_Step3_Passed_Elements.fasta，$TAG.result85，$TAG.outinner85")
    merge_dir = ltr_85_dir + "/merge"
    mkdir(merge_dir, True)
    step3_merge(ltr_85_dir, contig_list, merge_dir, 85)

    logger.info("   raw step 2.1.4 - 2.1.5")

    LTR85_lib_file = merge_dir + "/LTR85.lib"
    if os.path.exists(LTR85_lib_file):
        logger.info("   raw step 4-5, already finished, skip step")
    else:
        logger.info("   raw step 4-5, begin")
        step_4_5_85_sh = merge_dir + "/step_4_5.sh"
        step_4_5_85(step_4_5_85_sh, args.genome, args.trna_file, args.crl_dir, MITE_file, args.tpase_dir)
        code, output, error = cmd_run("bash %s" % step_4_5_85_sh, cwd=merge_dir)

        stdout_file = merge_dir + "/step_4_5.sh.o"
        with open(stdout_file, 'w') as f:
            f.write(output)

        stderr_file = merge_dir + "/step_4_5.sh.e"
        with open(stderr_file, 'w') as f:
            f.write(error)

        logger.info("   raw step 4-5, finished")

    logger.info("Step3.3: merge 85 and 99 lib")

    allLTR_file = merge_dir + "/allLTR.lib"
    if os.path.exists(allLTR_file):
        logger.info("   already finished, skip step")
    else:
        logger.info("   begin")
        copy_file(LTR99_lib_file, merge_dir)

        bash_file = merge_dir + "/merge85_99.sh"

        with open(bash_file, 'w') as f:
            f.write("""source activate maker_p && \\
RepeatMasker -pa 56 -lib LTR99.lib -dir . LTR85.lib && \\
perl %s/remove_masked_sequence.pl --masked_elements LTR85.lib.masked --outfile FinalLTR85.lib && \\
cat LTR99.lib FinalLTR85.lib > allLTR.lib
""" % args.crl_dir)

        code, output, error = cmd_run("bash %s" % bash_file, cwd=merge_dir)

        stdout_file = merge_dir + "/merge85_99.sh.o"
        with open(stdout_file, 'w') as f:
            f.write(output)

        stderr_file = merge_dir + "/merge85_99.sh.e"
        with open(stderr_file, 'w') as f:
            f.write(error)

        allLTR_file = merge_dir + "/allLTR.lib"

        logger.info("   finished")

    logger.info("Step4: Collecting repetitive sequences by RepeatModeler")

    repeat_dir = args.output_dir + "/Repeat"
    mkdir(repeat_dir, True)
    copy_file(allLTR_file, repeat_dir)
    copy_file(MITE_file, repeat_dir)

    cmd_run("cat allLTR.lib MITE.lib > allMITE_LTR.lib", cwd=repeat_dir)
    alllib_file = os.path.abspath(repeat_dir + "/allMITE_LTR.lib")

    logger.info("   RepeatMasker")

    tmp_out_pickle = repeat_dir + "/RepeatMasker.pyb"

    if os.path.exists(tmp_out_pickle):
        logger.info("   RepeatMasker, finished, skip step")
    else:
        bash_file = repeat_dir + "/RepeatMasker.sh"
        with open(bash_file, 'w') as f:
            f.write("""#! /bin/bash

source activate maker_p

GENOME=$1
TAG=$2
all_LIB=%s
thread=$3

mkdir ${TAG}
cd ${TAG}

RepeatMasker -pa $thread -lib $all_LIB -dir . $GENOME
""" % alllib_file)

        args_list = []
        for contig in contig_list:
            contig_fasta_file = os.path.abspath(split_genome_dir + "/" + contig + ".fa")
            if contig == 'genome':
                cmd_string = "bash %s %s %s %d" % (bash_file, contig_fasta_file, contig, args.num_threads)
            else:
                cmd_string = "bash %s %s %s %d" % (bash_file, contig_fasta_file, contig, 1)
            args_list.append((cmd_string, repeat_dir, 1, True, None))

        logger.info("   RepeatMasker, begin")

        tmp_out = multiprocess_running(cmd_run, args_list, args.num_threads, log_file=ltr_99_log)

        OUT = open(tmp_out_pickle, 'wb')
        pickle.dump(tmp_out, OUT)
        OUT.close()

        logger.info("   RepeatMasker, finished")

    logger.info("   RepeatModeler")

    ModelerID_lib_file = repeat_dir + "/ModelerID.lib"
    ModelerUnknown_lib_file = repeat_dir + "/ModelerUnknown.lib"

    if os.path.exists(ModelerID_lib_file):
        logger.info("   RepeatModeler, finished, skip step")
    else:
        bash_file = repeat_dir + "/RepeatModeler.sh"
        with open(bash_file, 'w') as f:
            f.write("""#! /bin/bash
CRL_DIR=%s
Tpase_dir=%s

source activate maker_p

cat */*.fa.masked > all.fa.masked
perl $CRL_DIR/rmaskedpart.pl all.fa.masked 50  >  all.fa.umseqfile
sed '/^$/d' all.fa.umseqfile > all.fa.fullline.umseqfile
BuildDatabase -name umseqfiledb -engine ncbi all.fa.fullline.umseqfile
RepeatModeler -pa 14 -database umseqfiledb

perl $CRL_DIR/repeatmodeler_parse.pl --fastafile RM*/consensi.fa.classified --unknowns repeatmodeler_unknowns.fasta --identities repeatmodeler_identities.fasta

# makeblastdb -in $Tpase_dir/Tpases020812  -dbtype prot
blastx -query repeatmodeler_unknowns.fasta -db $Tpase_dir/Tpases020812 -num_threads 56 -evalue 1e-10 -num_descriptions 10 -out modelerunknown_blast_results.txt
perl $CRL_DIR/transposon_blast_parse.pl --blastx modelerunknown_blast_results.txt --modelerunknown repeatmodeler_unknowns.fasta

mv  unknown_elements.txt  ModelerUnknown.lib
cat  identified_elements.txt  repeatmodeler_identities.fasta  > ModelerID.lib
""" % (args.crl_dir, args.tpase_dir))

        logger.info("   RepeatModeler, begin")
        code, output, error = cmd_run("bash %s" % bash_file, cwd=repeat_dir)

        stdout_file = repeat_dir + "/RepeatModeler.sh.o"
        with open(stdout_file, 'w') as f:
            f.write(output)

        stderr_file = repeat_dir + "/RepeatModeler.sh.e"
        with open(stderr_file, 'w') as f:
            f.write(error)

        logger.info("   RepeatModeler, finished")

    logger.info("Step5: Exclusion of gene fragments")

    no_gene_dir = args.output_dir + "/NoGeneFrag"
    mkdir(no_gene_dir, True)
    allRepeats_lib_file = no_gene_dir + "/allRepeats.lib"

    if os.path.exists(allRepeats_lib_file):
        logger.info("   Step5, finished, skip step")
    else:
        copy_file(allLTR_file, no_gene_dir)
        copy_file(MITE_file, no_gene_dir)
        copy_file(ModelerUnknown_lib_file, no_gene_dir)
        copy_file(ModelerID_lib_file, no_gene_dir)

        logger.info("   Blastx, round 1")
        code, output, error = cmd_run(
            "%s -num_threads 56 -query ModelerUnknown.lib -db %s -evalue 1e-10 -num_descriptions 10 -out ModelerUnknown.lib_blast_results.txt" % (
                args.old_blastx,
                args.protein_db
            ), cwd=no_gene_dir)

        stdout_file = no_gene_dir + "/blastx.r1.o"
        with open(stdout_file, 'w') as f:
            f.write(output)

        stderr_file = no_gene_dir + "/blastx.r1.e"
        with open(stderr_file, 'w') as f:
            f.write(error)

        logger.info("   Blastx, round 2")
        code, output, error = cmd_run(
            "%s -num_threads 56 -query MITE.lib -db %s -evalue 1e-10 -num_descriptions 10 -out MITE.lib_blast_results.txt" % (
                args.old_blastx,
                args.protein_db
            ), cwd=no_gene_dir)

        stdout_file = no_gene_dir + "/blastx.r2.o"
        with open(stdout_file, 'w') as f:
            f.write(output)

        stderr_file = no_gene_dir + "/blastx.r2.e"
        with open(stderr_file, 'w') as f:
            f.write(error)

        logger.info("   Blastx, round 3")
        code, output, error = cmd_run(
            "%s -num_threads 56 -query allLTR.lib -db %s -evalue 1e-10 -num_descriptions 10 -out allLTR.lib_blast_results.txt" % (
                args.old_blastx,
                args.protein_db
            ), cwd=no_gene_dir)

        stdout_file = no_gene_dir + "/blastx.r3.o"
        with open(stdout_file, 'w') as f:
            f.write(output)

        stderr_file = no_gene_dir + "/blastx.r3.e"
        with open(stderr_file, 'w') as f:
            f.write(error)

        logger.info("   Blastx, round 4")
        code, output, error = cmd_run(
            "%s -num_threads 56 -query ModelerID.lib -db %s -evalue 1e-10 -num_descriptions 10 -out ModelerID.lib_blast_results.txt" % (
                args.old_blastx,
                args.protein_db
            ), cwd=no_gene_dir)

        stdout_file = no_gene_dir + "/blastx.r4.o"
        with open(stdout_file, 'w') as f:
            f.write(output)

        stderr_file = no_gene_dir + "/blastx.r4.e"
        with open(stderr_file, 'w') as f:
            f.write(error)

        logger.info("   get final lib")
        bash_file = no_gene_dir + "/get_final_lib.sh"
        with open(bash_file, 'w') as f:
            f.write("""#! /bin/bash
DIR_PE=%s

source activate maker_p

$DIR_PE/ProtExcluder.pl -f 50 ModelerUnknown.lib_blast_results.txt ModelerUnknown.lib
$DIR_PE/ProtExcluder.pl -f 50 MITE.lib_blast_results.txt MITE.lib
$DIR_PE/ProtExcluder.pl -f 50 ModelerID.lib_blast_results.txt ModelerID.lib

sed 's/(//g;s/)//g' allLTR.lib > allLTR.lib.re
sed 's/_(/_/g;s/)_/_/g' allLTR.lib_blast_results.txt > allLTR.lib_re_blast_results.txt

perl $DIR_PE/ProtExcluder.pl -f 50 allLTR.lib_re_blast_results.txt allLTR.lib.re
cat MITE.libnoProtFinal allLTR.lib.renoProtFinal ModelerID.libnoProtFinal > KnownRepeats.lib
cat KnownRepeats.lib ModelerUnknown.libnoProtFinal > allRepeats.lib
""" % args.pe_dir)

        code, output, error = cmd_run("bash %s" % bash_file, cwd=no_gene_dir)

        stdout_file = no_gene_dir + "/get_final_lib.sh.o"
        with open(stdout_file, 'w') as f:
            f.write(output)

        stderr_file = no_gene_dir + "/get_final_lib.sh.e"
        with open(stderr_file, 'w') as f:
            f.write(error)

    copy_file(allRepeats_lib_file, args.output_dir)
    logger.info("Everything is finished")

def extract_maker_repeat_info():
    pass

# maker output



# gene name and description

# ncbi gene database
# wget https://ftp.ncbi.nih.gov/gene/DATA/gene_info.gz
gene_info_file = '/lustre/home/xuyuxing/Database/NCBI/gene/2020/gene_info.gz'

# wget https://ftp.ncbi.nih.gov/gene/DATA/gene2accession.gz
gene2accession = '/lustre/home/xuyuxing/Database/NCBI/gene/2020/gene2accession.gz'




import csv
import re


def trf_dat_to_dict(trf_dat):
    # trf_dat = "/lustre/home/xuyuxing/Database/Other_genome/Ath/Tair10/TAIR10_Chr.all.fasta.2.7.7.80.10.20.10.dat"
    fieldnames_list = ["start", "end", "period_size", "copy_num", "consensus_size", "matches", "indels", "score",
                       "A_perc", "C_perc", "G_perc", "T_perc", "entropy", "unit", "sequence"]
    trf_dict = {}
    with open(trf_dat, 'r') as dat:
        num = 0
        for info in dat.read().split("Sequence: "):
            if num == 0:
                num = num + 1
                continue
            info_dict = list(csv.DictReader(info.splitlines(), delimiter=" ", fieldnames=fieldnames_list))
            seqname = info.splitlines()[0]
            name_short = re.search('^(\S+)', seqname).group(1)
            del info_dict[0]
            del info_dict[0]
            trf_dict[name_short] = []
            for i in info_dict:
                i["chr"] = name_short
                if re.match(r'^\d+$', i["start"]):
                    i['start'] = int(i['start'])
                    i['end'] = int(i['end'])
                    i['period_size'] = int(i['period_size'])
                    i['copy_num'] = float(i['copy_num'])
                    i['consensus_size'] = int(i['consensus_size'])
                    i['matches'] = int(i['matches'])
                    i['indels'] = int(i['indels'])
                    i['score'] = int(i['score'])
                    i['A_perc'] = int(i['A_perc'])
                    i['C_perc'] = int(i['C_perc'])
                    i['G_perc'] = int(i['G_perc'])
                    i['T_perc'] = int(i['T_perc'])
                    i['entropy'] = float(i['entropy'])
                    trf_dict[name_short].append(i)
    return trf_dict



import os
import re
from copy import deepcopy
from lib.common.genome.genome_feature2 import ChrLoci, GenomeFeature, read_gff_file, cds_judgment, cluster_of_chr_loci, write_gff_file
from lib.common.genome.seq_base import read_fasta
from lib.common.os import mkdir, multiprocess_running
from api.common.genome.blast import blastn_pair_running
from lib.common.math.interval import merge_intervals
from interlap import InterLap
import pickle

def cDNA_coord_shift(give_site, direction, sub_feature_list):
    """
    give_site = 100
    direction = 'cDNA_to_Genome'
    sub_feature_list = mRNA.sub_features
    """

    strand = sub_feature_list[0].strand

    if strand == 1 or strand == "+":
        GC = sorted([(exon.start, exon.end) for exon in sub_feature_list if
                     exon.type == 'exon' or exon.type == 'match_part'])
    else:
        GC = sorted([(exon.end, exon.start) for exon in sub_feature_list if
                     exon.type == 'exon' or exon.type == 'match_part'], reverse=True)

    start = 0
    CC = []
    for i in GC:
        i_len = abs(i[1] - i[0]) + 1
        CC.append((start + 1, start + i_len))
        start = start + i_len

    if direction == 'cDNA_to_Genome':
        block_index = None
        for i in CC:
            if min(i) <= give_site <= max(i):
                block_index = CC.index(i)

        if block_index is None:
            raise ValueError("give_site is out of gene range")

        dela_len = give_site - CC[block_index][0]

        if strand == 1 or strand == "+":
            return GC[block_index][0] + dela_len
        else:
            return GC[block_index][0] - dela_len

    elif direction == 'Genome_to_cDNA':
        block_index = None
        for i in GC:
            if min(i) <= give_site <= max(i):
                block_index = GC.index(i)

        if block_index is None:
            raise ValueError("give_site is out of gene range")

        if strand == 1 or strand == "+":
            dela_len = give_site - GC[block_index][0]
        else:
            dela_len = GC[block_index][0] - give_site

        return CC[block_index][0] + dela_len

def cDNA_feature_coord(mRNA):
    output_list = []
    for ft in mRNA.sub_features:
        if ft.type == 'exon':
            continue

        ft_cDNA_start_tmp = cDNA_coord_shift(
            ft.start, "Genome_to_cDNA", mRNA.sub_features)
        ft_cDNA_end_tmp = cDNA_coord_shift(
            ft.end, "Genome_to_cDNA", mRNA.sub_features)

        ft_cDNA_start = min(ft_cDNA_start_tmp, ft_cDNA_end_tmp)
        ft_cDNA_end = max(ft_cDNA_start_tmp, ft_cDNA_end_tmp)

        output_list.append((ft.type, (ft_cDNA_start, ft_cDNA_end)))

    return output_list

def adjust_cds_phaes(cDNA_feature_list, phase):
    """
    cDNA_feature_list = [('five_prime_UTR',(1,20)),('CDS',(21,40)),('three_prime_UTR',(41,50))]
    cDNA_feature_list = [('five_prime_UTR',(1,10)),('CDS',(21,40)),('three_prime_UTR',(41,50))]
    phase = 1
    """

    for i in range(len(cDNA_feature_list)):
        type_tmp, range_tmp = cDNA_feature_list[i]
        if type_tmp == 'CDS':
            old_start = cDNA_feature_list[i][1][0]

            new_start = old_start + phase
            cDNA_feature_list[i] = (
                'CDS', (new_start, cDNA_feature_list[i][1][1]))

            if i > 0 and old_start - cDNA_feature_list[i - 1][1][1] == 1:
                new_end = new_start - 1
                cDNA_feature_list[i - 1] = (
                    cDNA_feature_list[i - 1][0], (cDNA_feature_list[i - 1][1][0], new_end))

            break

    return cDNA_feature_list

def no_gap_index(seq, gap_chr='-'):
    """0-base"""
    ind_list = []
    for i in range(len(seq)):
        c = seq[i]
        if i > 0:
            lc = seq[i - 1]

        # block start
        if i == 0 and c != gap_chr:
            blocking_start = 0
        if i != 0 and c != gap_chr and lc == gap_chr:
            blocking_start = i

        # block end
        if i != 0 and c == gap_chr and lc != gap_chr:
            blocking_end = i - 1
            ind_list.append((blocking_start, blocking_end))
        if i + 1 == len(seq):
            blocking_end = i
            ind_list.append((blocking_start, blocking_end))

    seq_index_interlap = InterLap()

    no_gap_end_index = -1

    for i in ind_list:
        seq_index_interlap.add(
            (no_gap_end_index + 1, i[1] - i[0] + 1 + no_gap_end_index, i))
        no_gap_end_index = i[1] - i[0] + 1 + no_gap_end_index

    return seq_index_interlap

def blast_hit_coord_shift(give_site, direction, blast_hsp):
    """
    give_site = 100 (1-base)
    direction = 'query_to_subject'
    blast_hsp = BlastHsp by xuyuxing
    """

    q_seq = blast_hsp.Hsp_qseq
    s_seq = blast_hsp.Hsp_hseq
    midline = blast_hsp.Hsp_midline

    q_from = blast_hsp.Hsp_query_from
    q_to = blast_hsp.Hsp_query_to
    s_from = blast_hsp.Hsp_hit_from
    s_to = blast_hsp.Hsp_hit_to

    q_iterlap = no_gap_index(q_seq)
    s_iterlap = no_gap_index(s_seq)

    if direction == 'query_to_subject':
        if give_site < q_from:
            give_site = q_from
        if give_site > q_to:
            give_site = q_to

        give_site_index = give_site - q_from

        s_start, s_end, (a_start, a_end) = list(
            q_iterlap.find((give_site_index, give_site_index)))[0]
        aln_site = give_site_index - s_start + a_start

        output_site = len(
            s_seq[0:aln_site + 1].replace("-", "")) + s_from - 1
    elif direction == 'subject_to_query':
        if give_site < s_from:
            give_site = s_from
        if give_site > s_to:
            give_site = s_to

        give_site_index = give_site - s_from

        s_start, s_end, (a_start, a_end) = list(
            s_iterlap.find((give_site_index, give_site_index)))[0]
        aln_site = give_site_index - s_start + a_start

        output_site = len(
            q_seq[0:aln_site + 1].replace("-", "")) + q_from - 1

    return output_site

def generate_new_feature(mRNA, phase, blat_cdna_seq, best_hsp, blat_hit):
    # read old cDNA structure
    feature_list = cDNA_feature_coord(mRNA)

    # adjust phase
    feature_list = adjust_cds_phaes(feature_list, phase)

    # map old cDNA to new cDNA
    new_cDNA_feature_list = []
    for type_tmp, range_tmp in feature_list:
        start_tmp = blast_hit_coord_shift(
            range_tmp[0], 'query_to_subject', best_hsp)
        end_tmp = blast_hit_coord_shift(
            range_tmp[1], 'query_to_subject', best_hsp)

        new_cDNA_feature_list.append((type_tmp, (start_tmp, end_tmp)))

    # remove bad feature
    new_cDNA_feature_filterd_list = []
    for type_tmp, range_tmp in new_cDNA_feature_list:
        if range_tmp[1] - range_tmp[0] == 0:
            continue
        new_cDNA_feature_filterd_list.append((type_tmp, range_tmp))

    # test cds is good
    cds_seq = ""
    for type_tmp, range_tmp in new_cDNA_feature_filterd_list:
        if not type_tmp == 'CDS':
            continue

        cds_seq += blat_cdna_seq[range_tmp[0] - 1:range_tmp[1]]

    good_orf, phase, aa_seq = cds_judgment(cds_seq)

    if good_orf:

        new_cDNA_feature_filterd_phased_list = adjust_cds_phaes(
            new_cDNA_feature_filterd_list, phase)

        # get new genome feature
        genome_feature_list = []
        for type_tmp, range_tmp in new_cDNA_feature_filterd_phased_list:
            start_tmp = cDNA_coord_shift(
                range_tmp[0], 'cDNA_to_Genome', blat_hit.sub_features)
            end_tmp = cDNA_coord_shift(
                range_tmp[1], 'cDNA_to_Genome', blat_hit.sub_features)

            # print(range_tmp,(start_tmp, end_tmp))

            genome_feature_list.append(
                (type_tmp, (start_tmp, end_tmp)))

        # exon get
        exon_range = []
        for type_tmp, range_tmp in genome_feature_list:
            exon_range.append(range_tmp)

        exon_range = merge_intervals(exon_range, True)

        for i in exon_range:
            genome_feature_list.append(('exon', i))

        # change to GenomeFeature
        named_genome_feature_list = []
        for type_name, type_tag in [('CDS', 'cds'), ('exon', 'exon'), ('five_prime_UTR', 'utr5p'),
                                    ('three_prime_UTR', 'utr3p')]:
            type_list = sorted(
                [i for i in genome_feature_list if i[0] == type_name], key=lambda x: x[1][0])
            for i in range(len(type_list)):
                named_genome_feature_list.append(
                    ("%s.%s%d" % (blat_hit.id, type_tag, i + 1), type_list[i][0], type_list[i][1]))

        sub_feature = []
        site_list = []
        for sub_gf_id, type_tmp, range_tmp in named_genome_feature_list:
            site_list.extend(list(range_tmp))
            sub_feature.append(GenomeFeature(id=sub_gf_id, type=type_tmp, chr_loci=ChrLoci(chr_id=blat_hit.chr_id, strand=blat_hit.strand, start=range_tmp[0], end=range_tmp[1])))

        # add cds phase
        cds_len_sum = 0
        if blat_hit.strand == '+' or blat_hit.strand == 1:
            for sub_gf in sorted(sub_feature, key=lambda x: x.start, reverse=False):
                if sub_gf.type == 'CDS':
                    if cds_len_sum % 3 == 0:
                        phase_tmp = 0
                    else:
                        phase_tmp = 3 - (cds_len_sum % 3)
                    sub_gf.phase = phase_tmp
                    sub_gf.qualifiers["phase"] = [sub_gf.phase]
                    sub_gf.qualifiers["phase2"] = phase_tmp
                    sub_gf.phase2 = sub_gf.phase
                    cds_len_sum += abs(sub_gf.start - sub_gf.end) + 1
        elif blat_hit.strand == '-' or blat_hit.strand == -1:
            for sub_gf in sorted(sub_feature, key=lambda x: x.start, reverse=True):
                if sub_gf.type == 'CDS':
                    plus_phase = 3 - (cds_len_sum % 3)
                    if plus_phase == 0:
                        phase_tmp = 0
                    else:
                        phase_tmp = 3 - plus_phase
                    sub_gf.phase = phase_tmp
                    # sub_gf.qualifiers["phase"] = [sub_gf.phase]
                    # sub_gf.qualifiers["phase2"] = phase_tmp
                    # sub_gf.phase2 = sub_gf.phase
                    cds_len_sum += abs(sub_gf.start - sub_gf.end) + 1

        mRNA_start = min(site_list)
        mRNA_end = max(site_list)

        blat_mRNA = GenomeFeature(id=blat_hit.id, type="mRNA", chr_loci=ChrLoci(chr_id=blat_hit.chr_id, strand=blat_hit.strand, start=mRNA_start, end=mRNA_end), sub_features=sub_feature)

        blat_mRNA.blat_hit = blat_hit
        blat_mRNA.good_cds = True

    else:
        blat_mRNA = GenomeFeature(id=blat_hit.id, type="mRNA", chr_loci=ChrLoci(chr_id=blat_hit.chr_id, strand=blat_hit.strand, start=0, end=0), sub_features=None)
        blat_mRNA.good_cds = False
        blat_mRNA.blat_hit = blat_hit

    return blat_mRNA


def hsp_judgment(blast_hsp):
    q_len = int(blast_hsp.query.qLen)
    s_len = int(blast_hsp.subject.Hit_len)

    if blast_hsp.Hsp_query_from == 1 and blast_hsp.Hsp_hit_from == 1 and q_len == blast_hsp.Hsp_query_to and s_len == blast_hsp.Hsp_hit_to and blast_hsp.Hsp_align_len == blast_hsp.Hsp_query_to and blast_hsp.Hsp_identical_site == blast_hsp.Hsp_query_to and blast_hsp.Hsp_gaps == 0:
        return True, 0, "perfect hit"

    elif blast_hsp.Hsp_identical_ratio < 0.95 or (
            blast_hsp.Hsp_query_to - blast_hsp.Hsp_query_from + 1) / q_len < 0.9:
        return False, 2, "low identity or coverage"

    else:
        return True, 1, "good hit"

def blat_hit_judgment(blat_cdna_seq, old_cdna_seq, tmp_dir):
    # print(blat_hit)

    blast_results = blastn_pair_running(('query', old_cdna_seq), ('subject', blat_cdna_seq),
                                        tmp_dir, False)

    if len(blast_results['query'].hit) < 1:
        return False, None, "No blast hit"

    good_hsp = []
    for hsp in blast_results['query'].hit[0].hsp:
        good_hsp_flag, note_code, note_hsp = hsp_judgment(hsp)
        if good_hsp_flag:
            good_hsp.append((hsp, note_code, note_hsp))

    if len(good_hsp) < 1:
        return False, None, "No good hit"

    best_hsp = sorted(good_hsp, key=lambda x: x[1])[0]

    return True, best_hsp[0], best_hsp[2]

def extract_give_feature_seq(give_type, sub_features_list, strand, genome_file=None, genome_dict=None):
    get_seq = ''

    if strand == 1 or strand == "+":
        feature_list = sorted([ft for ft in sub_features_list if ft.type == give_type],
                              key=lambda x: int(x.start), reverse=False)
    else:
        feature_list = sorted([ft for ft in sub_features_list if ft.type == give_type],
                              key=lambda x: int(x.start), reverse=True)

    for ft in feature_list:
        if genome_dict:
            get_seq += ft.get_sequence_quick(genome_dict).seq
        else:
            get_seq += ft.get_sequence(genome_file).seq

    return get_seq

# def map_old_gene_to_new_assembly(mRNA, blat_hit_list, old_chr_id, old_genome_fasta_file, new_genome_fasta_file,
#                                  tmp_dir, load_genome_into_mem=False):
def map_old_gene_to_new_assembly(mRNA, blat_hit_list, old_chr_id, tmp_dir):

    mRNA_tmp_dir = tmp_dir + "/" + mRNA.id
    mkdir(mRNA_tmp_dir, True)

    pyb_file = mRNA_tmp_dir + "/blat_mRNA.pyb"
    if not os.path.exists(pyb_file):

        # make sure old gene model is good, means cds is real ORF
        # if load_genome_into_mem:
        #     cds_seq = extract_give_feature_seq('CDS', mRNA.sub_features, mRNA.strand, genome_dict=old_genome_dict)
        # else:
        #     cds_seq = extract_give_feature_seq('CDS', mRNA.sub_features, mRNA.strand, genome_file=old_genome_fasta_file)
        cds_seq = mRNA.cds_seq

        good_orf, phase, aa_seq = cds_judgment(cds_seq)
        mRNA.good_cds = good_orf

        if good_orf:
            # get good blat hit
            # if load_genome_into_mem:
            #     old_cdna_seq = extract_give_feature_seq('exon', mRNA.sub_features, mRNA.strand, genome_dict=old_genome_dict)
            # else:
            #     old_cdna_seq = extract_give_feature_seq('exon', mRNA.sub_features, mRNA.strand, genome_file=old_genome_fasta_file)
            old_cdna_seq = mRNA.cdna_seq

            good_blat = []
            for blat_hit in blat_hit_list[:10]:
                # if load_genome_into_mem:
                #     blat_cdna_seq = extract_give_feature_seq("match_part", blat_hit.sub_features, blat_hit.strand, genome_dict=new_genome_dict)
                # else:
                #     blat_cdna_seq = extract_give_feature_seq("match_part", blat_hit.sub_features, blat_hit.strand, genome_file=new_genome_fasta_file)
                blat_cdna_seq = blat_hit.cdna_seq

                good_blat_flag, best_hsp, best_hsp_note = blat_hit_judgment(blat_cdna_seq,
                                                                            old_cdna_seq,
                                                                            mRNA_tmp_dir)

                blat_hit.hsp_note = best_hsp_note

                # print(best_hsp_note)

                if good_blat_flag:
                    good_blat.append(
                        (blat_hit, best_hsp, blat_cdna_seq))

            blat_mRNA_list = []
            for blat_hit, best_hsp, blat_cdna_seq in good_blat:
                blat_mRNA = generate_new_feature(
                    mRNA, phase, blat_cdna_seq, best_hsp, blat_hit)
                mRNA.chr_id = old_chr_id
                blat_mRNA.old_mRNA = mRNA
                blat_mRNA.best_hsp = best_hsp
                blat_mRNA_list.append(blat_mRNA)
        else:
            blat_mRNA_list = None

        OUT = open(pyb_file, 'wb')
        pickle.dump(blat_mRNA_list, OUT)
        OUT.close()

    else:
        TEMP = open(pyb_file, 'rb')
        blat_mRNA_list = pickle.load(TEMP)
        TEMP.close()

    return blat_mRNA_list

def blat_hit_load(blat_gff_file, feature_type):
    
    dict_tmp = {}

    gff_dict = read_gff_file(blat_gff_file)

    for type in gff_dict:
        for gf_id in gff_dict[type]:
            gf = gff_dict[type][gf_id]
            
            # single exon
            if gf.type != 'inferred_parent':
                new_gf = GenomeFeature(id=gf.qualifiers['Parent'][0], type='inferred_parent', chr_loci=gf.chr_loci, qualifiers={'ID': gf.qualifiers['Parent']}, sub_features=[gf])
                gf = new_gf

            # add new attr            
            gf.target = gf.sub_features[0].qualifiers['Target'][0].split()[0]
            gf.score = float(gf.sub_features[0].qualifiers['score'][0])
            gf.query_type = feature_type

            # parse sub_feature
            q_site_range = []
            for sf in gf.sub_features:
                target_info = sf.qualifiers['Target'][0].split()
                query_range = [int(target_info[1]), int(target_info[2])]
                sf.target = target_info[0]
                sf.q_start = min(query_range)
                sf.q_end = max(query_range)
                q_site_range.append(sf.q_start)
                q_site_range.append(sf.q_end)

            gf.q_start = min(q_site_range)
            gf.q_end = max(q_site_range)

            if gf.target not in dict_tmp:
                dict_tmp[gf.target] = []

            dict_tmp[gf.target].append(gf)

    for i in dict_tmp:
        dict_tmp[i] = sorted(
            dict_tmp[i], key=lambda x: x.score, reverse=True)

    # print(blat_gff_file + ".OK")
    return dict_tmp

def split_blat_gff(blat_gff_file, split_prefix, target_per_file):
    file_num = 0
    file_list = []
    with open(blat_gff_file, 'r') as f:
        last_t_id = ''
        save_target = 0
        for each_line in f:
            mbj = re.findall('Target=(\S+)', each_line)
            if len(mbj) == 0:
                continue
            t_id = mbj[0]
            if t_id != last_t_id:
                save_target += 1
                if save_target == target_per_file or file_num == 0:
                    if file_num > 0:
                        F1.close()
                    F1 = open("%s_%d.gff" % (split_prefix, file_num), 'w')
                    file_list.append("%s_%d.gff" % (split_prefix, file_num))
                    file_num += 1
                    save_target = 0
                last_t_id = t_id
            F1.write(each_line)
    return file_list

def split_and_load_blat_hit(raw_blat_gff_file, feature_type, split_dir, threads=80, target_per_file=1):
    mkdir(split_dir, False)

    split_file_list = split_blat_gff(raw_blat_gff_file, split_dir+"/split", target_per_file=target_per_file)
    # print('finished split')

    args_list = []
    for split_file in split_file_list:
        args_list.append((split_file, feature_type))

    mlt_out = multiprocess_running(blat_hit_load, args_list, threads, silence=False)

    dict_tmp = {}
    for i in mlt_out:
        for j in mlt_out[i]['output']:
            dict_tmp[j] = mlt_out[i]['output'][j]

    return dict_tmp

def get_cds_phase(gf):
    if gf.type == 'CDS':
        gf.qualifiers["phase"] = gf.phase
    if gf.sub_features and len(gf.sub_features) > 0:
        for sf in gf.sub_features:
            get_cds_phase(sf)

def MapOldAnno_main(args):

    if args.tmp_dir is None:
        args.tmp_dir = os.getcwd() + '/tmp'

    
    mkdir(args.tmp_dir, True)
    print("tmp dir is : %s" % args.tmp_dir)

    pyb_file = args.tmp_dir + "/data_load.pyb"
    if not os.path.exists(pyb_file):
        # read old gff file

        old_gff_dict = read_gff_file(args.old_gff)
        old_gene_gff = old_gff_dict['gene']

        # read cds and cDNA blat results
        split_dir = args.tmp_dir + "/split"
        cdna_dict = split_and_load_blat_hit(args.cDNA_gff, 'cDNA', split_dir, threads=args.num_threads, target_per_file=1)
        # cdna_dict = blat_hit_load(args.cDNA_gff, 'cDNA')

        for old_mRNA_id in cdna_dict:
            for i in range(len(cdna_dict[old_mRNA_id])):
                cdna_dict[old_mRNA_id][i].rank = i

        # cds_dict = blat_hit_load(args.CDS_gff, 'CDS')

        OUT = open(pyb_file, 'wb')
        pickle.dump((old_gene_gff, cdna_dict), OUT)
        OUT.close()
    else:
        TEMP = open(pyb_file, 'rb')
        old_gene_gff, cdna_dict = pickle.load(TEMP)
        TEMP.close()

    # just use cDNA and old gff
    blat_mRNA_dir = args.tmp_dir + "/blat_mRNA"
    mkdir(blat_mRNA_dir, True)

    old_genome_dict = read_fasta(args.old_genome)[0]
    new_genome_dict = read_fasta(args.new_genome)[0]

    # extract need seq:
    args_list = []
    for old_gene_id in old_gene_gff:
        # print(old_gene_id)
        gene = old_gene_gff[old_gene_id]
        for mRNA in gene.sub_features:
            if not mRNA.id in cdna_dict:
                continue

            cds_seq = extract_give_feature_seq('CDS', mRNA.sub_features, mRNA.strand, genome_dict=old_genome_dict)
            cdna_seq = extract_give_feature_seq('exon', mRNA.sub_features, mRNA.strand, genome_dict=old_genome_dict)
            mRNA.cds_seq = cds_seq
            mRNA.cdna_seq = cdna_seq

            blat_hit_list = []
            for blat_hit in cdna_dict[mRNA.id]:
                blat_cdna_seq = extract_give_feature_seq("match_part", blat_hit.sub_features, blat_hit.strand, genome_dict=new_genome_dict)
                blat_hit.cdna_seq = blat_cdna_seq
                blat_hit_list.append(blat_hit)

            args_list.append((mRNA, blat_hit_list, gene.chr_id, blat_mRNA_dir))

    map_output = multiprocess_running(
        map_old_gene_to_new_assembly, args_list, args.num_threads, silence=False)

    huge_blat_mRNA_list = []
    for i in map_output:
        if map_output[i]['output'] is None:
            continue
        for j in map_output[i]['output']:
            if j.good_cds:
                huge_blat_mRNA_list.append(j)

    blat_mRNA_cluster = cluster_of_chr_loci(huge_blat_mRNA_list, 0.5)

    write_blat_mRNA = []
    for contig in blat_mRNA_cluster:
        for strand in blat_mRNA_cluster[contig]:
            group_dict = blat_mRNA_cluster[contig][strand]
            for group_id in group_dict:
                gf_list = group_dict[group_id]['list']
                best_gf = sorted(
                    gf_list, key=lambda x: x.best_hsp.Hsp_score, reverse=True)[0]
                best_gf.used = True
                write_blat_mRNA.append(best_gf)

    write_blat_gene = []
    for mRNA in write_blat_mRNA:
        old_name = mRNA.id
        gene_name = old_name + ".gene"
        mRNA_name = old_name + ".mRNA"

        gene = GenomeFeature(id=gene_name, type='gene', chr_loci=mRNA.chr_loci)

        mRNA_sub_features = []
        for sub_mrna in mRNA.sub_features:
            sub_mrna2 = deepcopy(sub_mrna)
            if sub_mrna.type == 'CDS':
                sub_mrna2.qualifiers["phase"] = sub_mrna.phase
            mRNA_sub_features.append(sub_mrna2)

        # for sub_mrna in mRNA_sub_features:
        #     if sub_mrna.type == 'CDS':
        #         print(sub_mrna.qualifiers["phase"], sub_mrna.phase)

        mRNA.sub_features = mRNA_sub_features
        gene.sub_features = [mRNA]

        write_blat_gene.append(gene)

    write_gff_file(write_blat_gene, args.output_gff, source="blat", sort=True)
    # write_GenomeFeature_to_GFF(
    #     write_blat_gene, args.output_gff, source="blat")
    #
    # # write report
    # blat_mRNA_cluster_hash = {}
    # for contig in blat_mRNA_cluster:
    #     for strand in blat_mRNA_cluster[contig]:
    #         group_dict = blat_mRNA_cluster[contig][strand]
    #         for group_id in group_dict:
    #             group_range = group_dict[group_id]['range'][0]
    #             group_loci = ChrLoci(chr_id=contig, strand=strand, start=group_range[0], end=group_range[1])
    #             gf_list = group_dict[group_id]['list']
    #             for i in gf_list:
    #                 blat_mRNA_cluster_hash[i] = group_loci
    #
    # for i in map_output:
    #     mRNA, blat_hit_list, tmp1, tmp2, tmp3, tmp4 = map_output[i]['args']
    #     blat_mRNA_list = map_output[i]['output']
    #
    #     # old mRNA info
    #     if blat_mRNA_list is None:
    #         # old mRNA is not good cds
    #         old_cds_good = False
    #         blat_hit_passed_num = None
    #         blat_hit_total_num = len(cdna_dict[mRNA.id])
    #     else:
    #         old_cds_good = True
    #         blat_hit_passed_num = len(blat_mRNA_list)
    #         blat_hit_total_num = len(cdna_dict[mRNA.id])
    #
    #     # blat hit info
    #     if old_cds_good and blat_hit_passed_num > 0:
    #         mRNA = blat_mRNA_list[0].old_mRNA
    #         for blat_mRNA in blat_mRNA_list:
    #             blat_hit = blat_mRNA.blat_hit
    #             blat_hit_rank = blat_hit.rank
    #             blat_hit_note = blat_hit.hsp_note
    #             blat_cds_good = blat_mRNA.good_cds
    #             if hasattr(blat_mRNA, 'used'):
    #                 blat_used = blat_mRNA.used
    #             else:
    #                 blat_used = False
    #
    #             print(mRNA.id, old_cds_good, blat_hit_total_num, blat_hit_passed_num, blat_hit.name,blat_hit.get_fancy_name(),
    #                   blat_hit_rank, blat_hit_note, blat_cds_good, blat_used)


if __name__ == '__main__':
    ###
    class abc():
        pass

    args = abc()
    args.cDNA_gff = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phelipanche_aegyptiaca2/purge/MapOldAnno/all.cDNA.c1.0.gff"
    # args.cDNA_gff = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phelipanche_aegyptiaca2/purge/MapOldAnno/head.cDNA.c0.9.gff"
    args.old_gff = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phelipanche_aegyptiaca/T99112N0.genome.gff3"
    args.old_genome = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phelipanche_aegyptiaca/T99112N0.genome.fasta"
    args.new_genome = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phelipanche_aegyptiaca2/purge/purge_haplotigs/curated.fasta"
    args.tmp_dir = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phelipanche_aegyptiaca2/purge/MapOldAnno/tmp"
    args.num_threads = 160
    args.output_gff = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phelipanche_aegyptiaca2/purge/MapOldAnno/MapOldAnno.gff"

    MapOldAnno_main(args)


import re
from lib.common.genome.seq_base import read_fasta_by_faidx
from lib.common.genome.genome_feature2 import GenomeFeature, ChrLoci, write_gff_file
from lib.common.math.interval import merge_intervals

def MaskFasta2Gff_main(args):
    seq_dict = read_fasta_by_faidx(args.masked_file)

    gf_list = []
    for seq_id in seq_dict:
        print(seq_id)
        seq_now = seq_dict[seq_id].seq
        range_tmp_list = []
        for i in re.finditer('N', seq_now):
            range_tmp_list.append((i.end(), i.end()))
        range_list = merge_intervals(range_tmp_list, True)

        num = 0
        for i in range_list:
            gf = GenomeFeature(id="%s_masked_range_%d" % (seq_id, num), type='masked_range', chr_loci=ChrLoci(chr_id=seq_id, start=i[0], end=i[1]))
            gf_list.append(gf)
            num += 1

    write_gff_file(gf_list, args.output_gff, sort=True)

if __name__ == '__main__':
    class abc():
        pass

    args = abc()
    args.masked_file = "/lustre/home/xuyuxing/Database/Orobanche/annotation/EDTA/Ocu.genome.nt.1.0.fasta.mod.MAKER.masked.test"
    args.output_gff = "/lustre/home/xuyuxing/Database/Orobanche/annotation/EDTA/Ocu.genome.nt.1.0.fasta.mod.MAKER.masked.gff3"



# coding:utf-8

import matplotlib

matplotlib.use('Agg')

import matplotlib.pyplot as plt
from matplotlib.ticker import MultipleLocator
import numpy as np
import pylab
import matplotlib.patches as patches
import matplotlib.cbook as cbook
import Image
import matplotlib.pyplot as plt
import matplotlib.cbook as cbook
from pylab import *
import Image

img=Image.open('Cluster_0.U.seq.pdf')

with open('Cluster_0.U.seq.pdf', 'rb') as image_file:
    image = plt.imread(image_file)


fig, ax = plt.subplots()
im = ax.imshow(image)
patch = patches.Circle((260, 200), radius=200, transform=ax.transData)
im.set_clip_path(patch)

ax.axis('off')
plt.show()



import matplotlib.path as mpath
import matplotlib.patches as mpatches
import matplotlib.pyplot as plt
from matplotlib.path import Path
import matplotlib.patches as patches
import numpy as np
from lib.common.genome.genome_feature2 import sub_gf_traveler
from lib.common.math.interval import section
from matplotlib.patches import Rectangle

# gene structure plot



def gene_box(start, end, chr_line=0, box_width=1, **kwargs):
    if end < start:
        start, end = end, start
    
    rectangle_anchor_point = start,chr_line-0.5*box_width
    rectangle_width = end - start + 1
    rectangle_height = box_width
    
    return Rectangle(rectangle_anchor_point,rectangle_width,rectangle_height,**kwargs)
    

def add_box(ax, start, end, **kwargs):
    """
    use to plot a exon box on ax

    kwargs can have:
    horizon=0, width=1, facecolor='tab:blue', edgecolor='k', linewidth=2

    """

    defaults_kwargs = {
        'horizon': 0, 'vertical': None, 'width': 1, 'facecolor': 'tab:blue', 'edgecolor': 'k', 'linewidth': 2
    }

    for i in kwargs:
        if i in defaults_kwargs:
            defaults_kwargs[i] = kwargs[i]

    start, end = min(start, end), max(start, end)

    if not defaults_kwargs['vertical'] is None:
        path_data = [
            (mpath.Path.MOVETO, [
            defaults_kwargs['vertical'] + defaults_kwargs['width']/2, start]),
            (mpath.Path.LINETO, [
            defaults_kwargs['vertical'] + defaults_kwargs['width']/2, end]),
            (mpath.Path.LINETO, [
            defaults_kwargs['vertical'] - defaults_kwargs['width']/2, end]),
            (mpath.Path.LINETO, [
            defaults_kwargs['vertical'] - defaults_kwargs['width']/2, start]),
            (mpath.Path.CLOSEPOLY, [start, defaults_kwargs['vertical'] + defaults_kwargs['width']/2])]
    else:
        path_data = [
            (mpath.Path.MOVETO, [
            start, defaults_kwargs['horizon'] + defaults_kwargs['width']/2]),
            (mpath.Path.LINETO, [
            end, defaults_kwargs['horizon'] + defaults_kwargs['width']/2]),
            (mpath.Path.LINETO, [
            end, defaults_kwargs['horizon'] - defaults_kwargs['width']/2]),
            (mpath.Path.LINETO, [
            start, defaults_kwargs['horizon'] - defaults_kwargs['width']/2]),
            (mpath.Path.CLOSEPOLY, [start, defaults_kwargs['horizon'] + defaults_kwargs['width']/2])]
    

    codes, verts = zip(*path_data)
    path = mpath.Path(verts, codes)
    patch = mpatches.PathPatch(
        path, facecolor=defaults_kwargs['facecolor'], edgecolor=defaults_kwargs['edgecolor'], linewidth=defaults_kwargs['linewidth'])

    ax.add_patch(patch)

    return patch


def add_fat_arrow(ax, start, end, **kwargs):
    """
    use to plot a fat arrow to show gene direction

    kwargs can have:
    horizon=0, width=1, alpha=0.5, facecolor='tab:blue', edgecolor='k', linewidth=2, direction='right', arrow_head=0.5, arrow_head_length=None

    """

    defaults_kwargs = {
        'horizon': 0, 'width': 1, 'facecolor': 'tab:blue', 'edgecolor': 'k', 'linewidth': 2, "direction": 'right', "arrow_head": 0.5, "arrow_head_length": None
    }

    for i in kwargs:
        if i in defaults_kwargs:
            defaults_kwargs[i] = kwargs[i]

    if defaults_kwargs['arrow_head_length'] is None:
        defaults_kwargs['arrow_head_length'] = int(
            abs(start-end) * defaults_kwargs['arrow_head'])

    start, end = min(start, end), max(start, end)

    if defaults_kwargs['direction'] == "right":
        path_data = [
            (mpath.Path.MOVETO, [
             start, defaults_kwargs['horizon'] + defaults_kwargs['width'] / 2]),
            (mpath.Path.LINETO, [end - defaults_kwargs['arrow_head_length'],
                                 defaults_kwargs['horizon'] + defaults_kwargs['width'] / 2]),
            (mpath.Path.LINETO, [end, defaults_kwargs['horizon']]),
            (mpath.Path.LINETO, [end - defaults_kwargs['arrow_head_length'],
                                 defaults_kwargs['horizon'] - defaults_kwargs['width'] / 2]),
            (mpath.Path.LINETO, [
             start, defaults_kwargs['horizon'] - defaults_kwargs['width'] / 2]),
            (mpath.Path.CLOSEPOLY, [start, defaults_kwargs['horizon'] + defaults_kwargs['width'] / 2])]

    elif defaults_kwargs['direction'] == "left":
        path_data = [
            (mpath.Path.MOVETO, [
             end, defaults_kwargs['horizon'] + defaults_kwargs['width'] / 2]),
            (mpath.Path.LINETO, [start + defaults_kwargs['arrow_head_length'],
                                 defaults_kwargs['horizon'] + defaults_kwargs['width'] / 2]),
            (mpath.Path.LINETO, [start, defaults_kwargs['horizon']]),
            (mpath.Path.LINETO, [start + defaults_kwargs['arrow_head_length'],
                                 defaults_kwargs['horizon'] - defaults_kwargs['width'] / 2]),
            (mpath.Path.LINETO, [
             end, defaults_kwargs['horizon'] - defaults_kwargs['width'] / 2]),
            (mpath.Path.CLOSEPOLY, [end, defaults_kwargs['horizon'] + defaults_kwargs['width'] / 2])]

    codes, verts = zip(*path_data)
    path = mpath.Path(verts, codes)
    patch = mpatches.PathPatch(
        path, facecolor=defaults_kwargs['facecolor'], edgecolor=defaults_kwargs['edgecolor'], linewidth=defaults_kwargs['linewidth'])
    ax.add_patch(patch)

    return patch


def add_broken_line(ax, start, end, **kwargs):
    defaults_kwargs = {
        'horizon': 0, 'width': 1, 'edgecolor': 'k', 'linewidth': 2
    }

    for i in kwargs:
        if i in defaults_kwargs:
            defaults_kwargs[i] = kwargs[i]

    start, end = min(start, end), max(start, end)

    mid_site = int((end + start) / 2)
    x = np.array([start, mid_site, end])
    y = np.array([defaults_kwargs['horizon'], defaults_kwargs['horizon'] +
                  defaults_kwargs['width']/2, defaults_kwargs['horizon']])
    ax.plot(x, y, color=defaults_kwargs['edgecolor'],
            linewidth=defaults_kwargs['linewidth'])


def mRNA_structure_plot(ax, mRNA, **kwargs):
    """
    use to plot a fat arrow to show gene direction

    kwargs can have:
    horizon=0, width=1, facecolor='tab:blue', edgecolor='k', linewidth=2, utr_alpha=0.5, relative_flag=False

    """

    defaults_kwargs = {
        'horizon': 0, 'width': 1, 'cds_color': '#4F94CD', 'utr_color': '#CAE1FF', 'edgecolor': '#316399', 'linewidth': 2, 'relative_flag': False, 'plot_utr': True}

    for i in kwargs:
        if i in defaults_kwargs:
            defaults_kwargs[i] = kwargs[i]

    mRNA.get_introns()
    if defaults_kwargs['plot_utr']:
        utr_list = sorted([i for i in sub_gf_traveler(mRNA) if i.type in [
            'five_prime_UTR', 'three_prime_UTR', 'five_prime_utr', 'three_prime_utr']], key=lambda x: x.start, reverse=(mRNA.strand == '-'))
    else:
        utr_list = []
    cds_list = sorted([i for i in sub_gf_traveler(mRNA) if i.type == 'CDS'], key=lambda x: x.start, reverse=(mRNA.strand == '-'))
    intron_list = sorted([i for i in sub_gf_traveler(mRNA) if i.type == 'intron'], key=lambda x: x.start, reverse=(mRNA.strand == '-'))

    if mRNA.strand == '+':
        last_box = sorted(utr_list + cds_list,
                          key=lambda x: x.start, reverse=True)[0]
        mRNA_start = min([i.start for i in utr_list + cds_list] + [i.end for i in utr_list + cds_list])
        mRNA_end = max([i.start for i in utr_list + cds_list] + [i.end for i in utr_list + cds_list])
    else:
        last_box = sorted(utr_list + cds_list,
                          key=lambda x: x.start, reverse=False)[0]
        mRNA_start = max([i.start for i in utr_list + cds_list] + [i.end for i in utr_list + cds_list])
        mRNA_end = min([i.start for i in utr_list + cds_list] + [i.end for i in utr_list + cds_list])

    # print(mRNA_start)

    for utr in utr_list:
        if utr == last_box:
            continue
        if defaults_kwargs['relative_flag']:
            add_box(ax, abs(utr.start - mRNA_start) + 1, abs(utr.end - mRNA_start) + 1, horizon=defaults_kwargs['horizon'], width=defaults_kwargs['width'],
                    facecolor=defaults_kwargs['utr_color'], edgecolor=defaults_kwargs['edgecolor'], linewidth=defaults_kwargs['linewidth'])            
        else:
            add_box(ax, utr.start, utr.end, horizon=defaults_kwargs['horizon'], width=defaults_kwargs['width'],
                    facecolor=defaults_kwargs['utr_color'], edgecolor=defaults_kwargs['edgecolor'], linewidth=defaults_kwargs['linewidth'])

    for cds in cds_list:
        if cds == last_box:
            continue
        if defaults_kwargs['relative_flag']:
            add_box(ax, abs(cds.start - mRNA_start) + 1, abs(cds.end - mRNA_start) + 1, horizon=defaults_kwargs['horizon'], width=defaults_kwargs['width'],
                facecolor=defaults_kwargs['cds_color'], edgecolor=defaults_kwargs['edgecolor'], linewidth=defaults_kwargs['linewidth'])
        else:
            add_box(ax, cds.start, cds.end, horizon=defaults_kwargs['horizon'], width=defaults_kwargs['width'],
                facecolor=defaults_kwargs['cds_color'], edgecolor=defaults_kwargs['edgecolor'], linewidth=defaults_kwargs['linewidth'])

    for intron in intron_list:
        if_flag, deta = section(intron.range, (mRNA_start, mRNA_end))
        if if_flag and abs(deta[1] - deta[0]) > 0:
            if defaults_kwargs['relative_flag']:
                add_broken_line(ax, abs(intron.start - mRNA_start) + 1, abs(intron.end - mRNA_start) + 1, horizon=defaults_kwargs['horizon'], width=defaults_kwargs[
                            'width']/2, edgecolor=defaults_kwargs['edgecolor'], linewidth=defaults_kwargs['linewidth'])
            else:
                add_broken_line(ax, intron.start, intron.end, horizon=defaults_kwargs['horizon'], width=defaults_kwargs[
                            'width']/2, edgecolor=defaults_kwargs['edgecolor'], linewidth=defaults_kwargs['linewidth'])

    if last_box in utr_list:
        last_color = defaults_kwargs['utr_color']
    else:
        last_color = defaults_kwargs['cds_color']

    if mRNA.strand == '+':
        direction = 'right'
    elif mRNA.strand == '-':
        direction = 'left'

    arrow_head_length = min(sum(
        [abs(i.start - i.end) + 1 for i in utr_list + cds_list]) * 0.1, abs(last_box.start - last_box.end) + 1 * 0.5)

    if defaults_kwargs['relative_flag']:
        add_fat_arrow(ax, abs(last_box.start - mRNA_start) + 1, abs(last_box.end - mRNA_start) + 1, horizon=defaults_kwargs['horizon'], width=defaults_kwargs['width'], facecolor=last_color,
                  edgecolor=defaults_kwargs['edgecolor'], linewidth=defaults_kwargs['linewidth'], direction='right', arrow_head_length=arrow_head_length)
    else:
        add_fat_arrow(ax, last_box.start, last_box.end, horizon=defaults_kwargs['horizon'], width=defaults_kwargs['width'], facecolor=last_color,
                  edgecolor=defaults_kwargs['edgecolor'], linewidth=defaults_kwargs['linewidth'], direction=direction, arrow_head_length=arrow_head_length)


def collinear_bar(ax, A_range, B_range, facecolor='orange', alpha=0.3, A_y=2, A_y_c=1, B_y=-2, B_y_c=-1):

    A_start, A_end = A_range
    A_y = 2
    A_y_c = 1
    B_start, B_end = B_range
    B_y = -2
    B_y_c = -1

    verts = [
        (A_start, A_y),
        (A_end, A_y),
        (A_end, A_y_c),
        (B_end, B_y_c),
        (B_end, B_y),
        (B_start, B_y),
        (B_start, B_y_c),
        (A_start, A_y_c),
        (A_start, A_y),
        (A_start, A_y),
    ]

    codes = [
        Path.MOVETO,
        Path.LINETO,
        Path.CURVE4,
        Path.CURVE4,
        Path.CURVE4,
        Path.LINETO,
        Path.CURVE4,
        Path.CURVE4,
        Path.CURVE4,
        Path.CLOSEPOLY,
    ]

    path = Path(verts, codes)

    patch = patches.PathPatch(path, facecolor=facecolor, alpha=alpha, lw=0)
    ax.add_patch(patch)


if __name__ == "__main__":
    from lib.common.genome.genome_feature2 import mRNA, read_gff_file

    gff_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Gastrodia_elata/T91201N0.genome.gff3'
    gff_dict = read_gff_file(gff_file)
    gene_dict = gff_dict['gene']

    gf = gene_dict['GelC19G00020']

    # plot
    fig, ax = plt.subplots(figsize=(50,10))

    mRNA_gf = mRNA(from_gf=gf.sub_features[0])

    mRNA_structure_plot(ax, mRNA_gf)
    mRNA_structure_plot(ax, mRNA_gf, horizon=0, relative_flag=True)

    ax.set_xlim(mRNA_gf.start, mRNA_gf.end)

    ax.set_ylim(-5, 5)    
        
    plt.show()    



from lib.common.genome.genome_feature2 import ChrLoci, Gene, Genome, Chromosome
from lib.common.math.interval import section
from collections import OrderedDict
from lib.xuyuxing.base.common_command import get_od


class GenePair(object):
    def __init__(self, q_gene, s_gene, property_dict=None, gene_pair_id=None):
        self.q_gene = q_gene
        self.s_gene = s_gene
        self.property = property_dict
        self.id = gene_pair_id

    def reverse_myself(self):
        new_GP = GenePair(self.s_gene, self.q_gene, self.property)
        return new_GP

    def __str__(self):
        return "%s vs %s" % (self.q_gene, self.s_gene)

    __repr__ = __str__

    def __eq__(self, other):
        return (self.q_gene == other.q_gene and self.s_gene == other.s_gene) or (self.q_gene == other.s_gene and self.s_gene == other.q_gene)


class LociPair(object):
    def __init__(self, q_chr_loci, s_chr_loci, strand=None, loci_pair_id=None):
        self.q_chr_loci = q_chr_loci
        self.s_chr_loci = s_chr_loci
        self.strand = strand
        self.id = loci_pair_id

    def reverse_myself(self, new_loci_pair_id=None):
        new_lp = LociPair(self.s_chr_loci, self.q_chr_loci,
                          self.strand, self.new_loci_pair_id)
        return new_lp

    def __str__(self):
        return "Q = %s vs S = %s, %s" % (self.q_chr_loci, self.s_chr_loci, self.strand)

    __repr__ = __str__

    def __eq__(self, other):
        return (self.q_chr_loci == other.q_chr_loci and self.s_chr_loci == other.s_chr_loci) or (self.q_chr_loci == other.s_chr_loci and self.s_chr_loci == other.q_chr_loci)


def sort_gene_list_by_chr_rank(gene_list):
    return sorted(gene_list, key=lambda x: x.chr_rank)


class SyntenyBlock(LociPair):
    def __init__(self, q_gene_list=None, s_gene_list=None, anchored_gene_pair_list=None, property_dict=None, parameter_dict=None, synteny_block_id=None):
        # sort gene
        q_gene_list = sort_gene_list_by_chr_rank(q_gene_list)
        s_gene_list = sort_gene_list_by_chr_rank(s_gene_list)

        first_q_gene_rank = min([i.chr_rank for i in q_gene_list])
        last_q_gene_rank = min([i.chr_rank for i in q_gene_list])
        first_s_gene_rank = min([i.chr_rank for i in s_gene_list])
        last_s_gene_rank = min([i.chr_rank for i in s_gene_list])

        q_genes = OrderedDict()
        for gene in q_gene_list:
            q_genes[gene.id] = gene

        s_genes = OrderedDict()
        for gene in s_gene_list:
            s_genes[gene.id] = gene

        # if all anchored gene in gene list
        for gp in anchored_gene_pair_list:
            if not (gp.q_gene.id in q_genes):
                raise ValueError(
                    "find a query anchored gene (%s) not in gene list" % gp.q_gene.id)
            if not (gp.s_gene.id in s_genes):
                raise ValueError(
                    "find an subject anchored gene (%s) not in gene list" % gp.s_gene.id)

        # sort anchored_gene_pair_list by query rank
        anchored_gene_pair_list_sorted = sorted(
            anchored_gene_pair_list, key=lambda x: x.q_gene.chr_rank)

        # get q & s chr_loci
        q_chr_id = q_gene_list[0].chr_loci.chr_id
        q_sp_id = q_gene_list[0].chr_loci.sp_id
        q_start = min([i.start for i in q_gene_list] +
                      [i.end for i in q_gene_list])
        q_end = max([i.start for i in q_gene_list] +
                    [i.end for i in q_gene_list])
        q_chr_loci = ChrLoci(chr_id=q_chr_id, strand=None,
                             start=q_start, end=q_end, species_id=q_sp_id)
        q_chr_loci.get_fancy_name()

        s_chr_id = s_gene_list[0].chr_loci.chr_id
        s_sp_id = s_gene_list[0].chr_loci.sp_id
        s_start = min([i.start for i in s_gene_list] +
                      [i.end for i in s_gene_list])
        s_end = max([i.start for i in s_gene_list] +
                    [i.end for i in s_gene_list])
        s_chr_loci = ChrLoci(chr_id=s_chr_id, strand=None,
                             start=s_start, end=s_end, species_id=s_sp_id)
        s_chr_loci.get_fancy_name()

        # get strand
        first_gp = [
            i for i in anchored_gene_pair_list_sorted if i.q_gene.chr_rank == first_q_gene_rank][0]
        if first_gp.s_gene.chr_rank == first_s_gene_rank:
            strand = "+"
        elif first_gp.s_gene.chr_rank == last_s_gene_rank:
            strand = "-"
        else:
            raise ValueError("failed to get strand")

        # change to LociPair
        super(SyntenyBlock, self).__init__(q_chr_loci, s_chr_loci,
                                           strand=strand, loci_pair_id=synteny_block_id)

        self.id = synteny_block_id
        self.property_dict = property_dict
        self.parameter_dict = parameter_dict

        # add other information
        self.q_genes = q_genes
        self.s_genes = s_genes
        self.anchored_gene_pairs = anchored_gene_pair_list_sorted
        self.q_sp_id = q_sp_id
        self.s_sp_id = s_sp_id
        self.q_chr = q_chr_id
        self.s_chr = s_chr_id

    def reverse_myself(self, new_sb_id):
        s_gene_list = [self.q_genes[i] for i in self.q_genes]
        q_gene_list = [self.s_genes[i] for i in self.s_genes]

        anchored_gene_pair_list = []
        for gp in self.anchored_gene_pair_list:
            re_gp = gp.reverse_myself()
            anchored_gene_pair_list.append(re_gp)

        new_sb = SyntenyBlock(q_gene_list=q_gene_list, s_gene_list=s_gene_list, anchored_gene_pair_list=anchored_gene_pair_list,
                              property_dict=self.property_dict, parameter_dict=self.parameter_dict, synteny_block_id=new_sb_id)

        return new_sb

    def __str__(self):
        q_s = get_od(self.q_genes, 0).chr_rank
        q_e = get_od(self.q_genes, -1).chr_rank

        if self.strand == "-":
            s_s = get_od(self.s_genes, 0).chr_rank
            s_e = get_od(self.s_genes, -1).chr_rank
        else:
            s_s = get_od(self.s_genes, -1).chr_rank
            s_e = get_od(self.s_genes, 0).chr_rank

        return "Q = %s:%s gene: %d-%d (%d) base: %d-%d (%d) vs S = %s:%s gene: %d-%d (%d) base: %d-%d (%d), %s, have %d gene pair" % (
            self.q_sp_id,
            self.q_chr_id,
            q_s,
            q_e,
            q_e - q_s + 1,
            self.q_chr_loci.start,
            self.q_chr_loci.end,
            self.q_chr_loci.end - self.q_chr_loci.start + 1,
            self.s_sp_id,
            self.s_chr_id,
            s_s,
            s_e,
            s_e - s_s + 1,
            self.s_chr_loci.start,
            self.s_chr_loci.end,
            self.s_chr_loci.end - self.s_chr_loci.start + 1,
            self.strand,
            len(self.anchored_gene_pairs)
        )

    __repr__ = __str__


def section_LP(loci_pair_a, loci_pair_b, section_side=2):
    a_q_chr_loci = loci_pair_a.q_chr_loci
    a_s_chr_loci = loci_pair_a.s_chr_loci
    b_q_chr_loci = loci_pair_b.q_chr_loci
    b_s_chr_loci = loci_pair_b.s_chr_loci

    if section_side == 2:
        if a_q_chr_loci.sp_id == b_q_chr_loci.sp_id and a_q_chr_loci.chr_id == b_q_chr_loci.chr_id and a_s_chr_loci.sp_id == b_s_chr_loci.sp_id and a_s_chr_loci.chr_id == b_s_chr_loci.chr_id:
            if section(a_q_chr_loci.range, b_q_chr_loci.range, just_judgement=True) and section(a_s_chr_loci.range, b_s_chr_loci.range, just_judgement=True):
                return True
        elif a_q_chr_loci.sp_id == b_s_chr_loci.sp_id and a_q_chr_loci.chr_id == b_s_chr_loci.chr_id and a_s_chr_loci.sp_id == b_q_chr_loci.sp_id and a_s_chr_loci.chr_id == b_q_chr_loci.chr_id:
            if section(a_q_chr_loci.range, b_s_chr_loci.range, just_judgement=True) and section(a_q_chr_loci.range, b_s_chr_loci.range, just_judgement=True):
                return True

        return False

    elif section_side == 1:
        if a_q_chr_loci.sp_id == b_q_chr_loci.sp_id and a_q_chr_loci.chr_id == b_q_chr_loci.chr_id and section(a_q_chr_loci.range, b_q_chr_loci.range, just_judgement=True):
            return True

        if a_s_chr_loci.sp_id == b_s_chr_loci.sp_id and a_s_chr_loci.chr_id == b_s_chr_loci.chr_id and section(a_s_chr_loci.range, b_s_chr_loci.range, just_judgement=True):
            return True

        if a_q_chr_loci.sp_id == b_s_chr_loci.sp_id and a_q_chr_loci.chr_id == b_s_chr_loci.chr_id and section(a_q_chr_loci.range, b_s_chr_loci.range, just_judgement=True):
            return True

        if a_s_chr_loci.sp_id == b_q_chr_loci.sp_id and a_s_chr_loci.chr_id == b_q_chr_loci.chr_id and section(a_q_chr_loci.range, b_s_chr_loci.range, just_judgement=True):
            return True

        return False


def merge_two_loci_pair(loci_pair_a, loci_pair_b, new_lp_id=None):
    if not section_LP(loci_pair_a, loci_pair_b, 2):
        raise ValueError(
            "given loci pair is failed to get section in two side")

    a_q_chr_loci = loci_pair_a.q_chr_loci
    a_s_chr_loci = loci_pair_a.s_chr_loci
    b_q_chr_loci = loci_pair_b.q_chr_loci
    b_s_chr_loci = loci_pair_b.s_chr_loci

    q_chr = a_q_chr_loci.chr_id
    q_sp_id = a_q_chr_loci.sp_id
    s_chr = a_s_chr_loci.chr_id
    s_sp_id = a_s_chr_loci.sp_id

    if a_q_chr_loci.sp_id == b_q_chr_loci.sp_id and a_q_chr_loci.chr_id == b_q_chr_loci.chr_id and section(a_q_chr_loci.range, b_q_chr_loci.range, just_judgement=True):
        q_start = min([a_q_chr_loci.start, a_q_chr_loci.end] +
                      [b_q_chr_loci.start, b_q_chr_loci.end])
        q_end = max([a_q_chr_loci.start, a_q_chr_loci.end] +
                    [b_q_chr_loci.start, b_q_chr_loci.end])
        s_start = min([a_s_chr_loci.start, a_s_chr_loci.end] +
                      [b_s_chr_loci.start, b_s_chr_loci.end])
        s_end = max([a_s_chr_loci.start, a_s_chr_loci.end] +
                    [b_s_chr_loci.start, b_s_chr_loci.end])

    elif a_q_chr_loci.sp_id == b_s_chr_loci.sp_id and a_q_chr_loci.chr_id == b_s_chr_loci.chr_id and section(a_q_chr_loci.range, b_s_chr_loci.range, just_judgement=True):
        q_start = min([a_q_chr_loci.start, a_q_chr_loci.end] +
                      [b_s_chr_loci.start, b_s_chr_loci.end])
        q_end = max([a_q_chr_loci.start, a_q_chr_loci.end] +
                    [b_s_chr_loci.start, b_s_chr_loci.end])
        s_start = min([a_s_chr_loci.start, a_s_chr_loci.end] +
                      [b_q_chr_loci.start, b_q_chr_loci.end])
        s_end = max([a_s_chr_loci.start, a_s_chr_loci.end] +
                    [b_q_chr_loci.start, b_q_chr_loci.end])

    else:
        raise ValueError(
            "given loci pair is failed to get section in two side")

    q_chr_loci = ChrLoci(q_chr, None, q_start, q_end, q_sp_id)
    s_chr_loci = ChrLoci(s_chr, None, s_start, s_end, s_sp_id)

    if a_q_chr_loci.len() > b_q_chr_loci.len():
        strand = loci_pair_a.strand
    else:
        strand = loci_pair_b.strand

    new_lp = LociPair(q_chr_loci, s_chr_loci,
                      strand=strand, loci_pair_id=new_lp_id)

    return new_lp


def merge_loci_pair_list(loci_pair_list):
    # print(len(loci_pair_list))
    # print(loci_pair_list)
    loci_pair_dict = dict(enumerate(loci_pair_list))

    merged_flag = False
    merged_id_list = []
    merged_loci_pair_list = []
    for i in loci_pair_dict:
        if merged_flag:
            break
        for j in loci_pair_dict:
            if merged_flag:
                break
            if i == j:
                continue
            else:
                if section_LP(loci_pair_dict[i], loci_pair_dict[j], 2):
                    merged_lp = merge_two_loci_pair(
                        loci_pair_dict[i], loci_pair_dict[j])
                    merged_loci_pair_list.append(merged_lp)
                    merged_flag = True
                    merged_id_list.append(i)
                    merged_id_list.append(j)

    if merged_flag:
        output_loci_pair_list = []
        for i in loci_pair_dict:
            if not i in merged_id_list:
                output_loci_pair_list.append(loci_pair_dict[i])

        output_loci_pair_list = output_loci_pair_list + merged_loci_pair_list
        output_loci_pair_list = merge_loci_pair_list(output_loci_pair_list)
        return output_loci_pair_list
    else:
        return loci_pair_list


if __name__ == "__main__":
    # test

    q_cl = ChrLoci(chr_id='Chr1', start=1000000, end=2000000, species_id='Sly')
    s_cl = ChrLoci(chr_id='Chr6', start=11000000,
                   end=12000000, species_id='Stu')

    lp1 = LociPair(q_cl, s_cl, strand="+", loci_pair_id=1)

    q_cl = ChrLoci(chr_id='Chr1', start=1500000, end=1600000, species_id='Sly')
    s_cl = ChrLoci(chr_id='Chr6', start=11500000,
                   end=11600000, species_id='Stu')

    lp2 = LociPair(q_cl, s_cl, strand="-", loci_pair_id=2)

    q_cl = ChrLoci(chr_id='Chr1', start=150000, end=160000, species_id='Sly')
    s_cl = ChrLoci(chr_id='Chr6', start=1150000, end=1160000, species_id='Stu')

    lp3 = LociPair(q_cl, s_cl, strand="-", loci_pair_id=2)

    q_cl = ChrLoci(chr_id='Chr1', start=2500000, end=2600000, species_id='Sly')
    s_cl = ChrLoci(chr_id='Chr6', start=21500000,
                   end=21600000, species_id='Stu')

    lp4 = LociPair(q_cl, s_cl, strand="+", loci_pair_id=2)

    q_cl = ChrLoci(chr_id='Chr1', start=1550000, end=2150000, species_id='Sly')
    s_cl = ChrLoci(chr_id='Chr6', start=11550000,
                   end=11650000, species_id='Stu')

    # lp5 = LociPair(q_cl, s_cl, strand="+", loci_pair_id=2)
    lp5 = LociPair(s_cl, q_cl, strand="+", loci_pair_id=2)

    lp_list = [lp1, lp2, lp3, lp4, lp5]

    merged_lp_list = merge_loci_pair_list(lp_list)

    # merge test
    from api.xuyuxing.file_parser.fileIO import tsv_file_parse

    synteny_file = '/lustre/home/xuyuxing/Work/Gel/GenomeStat/Gel_synteny_by_Ash.txt'
    file_info = tsv_file_parse(synteny_file)

    lp_list = []
    for i in file_info:
        q_sp, q_c, q_s, q_e, s_sp, s_c, s_s, s_e, strand = file_info[i]
        q_sp, q_c, q_s, q_e, s_sp, s_c, s_s, s_e, strand = q_sp, q_c, int(q_s), int(q_e), s_sp, s_c, int(s_s), int(s_e), strand
        
        q_cl = ChrLoci(chr_id=q_c, start=q_s, end=q_e, species_id=q_sp)
        s_cl = ChrLoci(chr_id=s_c, start=s_s, end=s_e, species_id=s_sp)

        lp = LociPair(q_cl, s_cl, strand=strand)

        lp_list.append(lp)

    merged_lp_list = merge_loci_pair_list(lp_list)

    # coverage
    


import os
import re
import shutil
from config import treeshrink_path
from lib.common.os import cmd_run


def treeshrink_running(tree_file, outdir, q_value=0.05, keep=False):
    if os.path.exists(outdir):
        shutil.rmtree(outdir)

    cmd_string = "%s -t %s -q %s -o %s" % (treeshrink_path, tree_file, str(q_value), outdir)
    cmd_run(cmd_string, silence=True)

    shrunk_list = []
    for file in os.listdir(outdir):
        match_Obj = re.match('^.*\_%s\.txt$' % str(q_value), file)
        if match_Obj:
            with open(outdir + "/" + file, 'r') as f:
                for each_line in f:
                    each_line = each_line.strip()
                    info = each_line.split("\t")
                    shrunk_list.extend(info)

    shrunk_list = list(set(shrunk_list))

    if not keep:
        shutil.rmtree(outdir)

    return shrunk_list





"""
This module work for parse NCBI taxonomy database.

wget ftp://ftp.ncbi.nlm.nih.gov/pub/taxonomy/taxdump.tar.gz

usage:
#
from taxonomy_lineage import Taxon,build_taxon_database
tax_record_dict = build_taxon_database(names_dmp_file,nodes_dmp_file)
#
class:
    Taxon
function:
    build_taxon_database
parsed output in main:
    a tsv file with two column: tax_id,lineage_list
log:
    Yuxing Xu
    2018.01.17  rewrite module from old script by Yuxing Xu
"""

__author__ = 'Yuxing Xu'




if __name__ == '__main__':
    import argparse

    ###### argument parse
    parser = argparse.ArgumentParser(
        prog='TaxonTools',
    )

    subparsers = parser.add_subparsers(title='subcommands', dest="subcommand_name")

    # argparse for StoreTaxonDB
    parser_a = subparsers.add_parser('StoreTaxonDB',
                                     help='store a taxon database into a sqlite3', description='')
    parser_a.add_argument("tax_dir", help="Path for uncompressed taxdump.tar.gz", type=str)
    parser_a.add_argument("tax_db_file", help="Path for sqlite3 file", type=str)

    # argparse for ID2Lineage
    parser_a = subparsers.add_parser('ID2Lineage',
                                     help='get taxonomy lineage by taxon ID', description='')
    parser_a.add_argument("tax_dir", help="Path for uncompressed taxdump.tar.gz", type=str)
    parser_a.add_argument("-q", "--query_tax_id_file", help="The file containing the tax_id to query (default as all taxon\
                         in database)", default=None, type=str)
    parser_a.add_argument("-o", "--output_file", help="Output file name (default as $STDOUT)", default=None, type=str)

    # argparse for Name2Lineage
    parser_a = subparsers.add_parser('Name2Lineage',
                                     help='get taxonomy lineage by taxon science name', description='')

    parser_a.add_argument("query_file", help="The file containing the tax name to query", type=str)
    parser_a.add_argument("tax_dir", help="Path for uncompressed taxdump.tar.gz", type=str)
    parser_a.add_argument("-o", "--output_file", help="Output file name (default as $STDOUT)", default=None, type=str)
    parser_a.add_argument("-f", "--fuzzy_search", help="use fuzzy search for more time", action='store_true')

    # argparse for RankStat
    parser_a = subparsers.add_parser('RankStat',
                                     help='to stat a rank taxonomy under given taxonmoy, such as return all order under fungi')

    parser_a.add_argument("tax_dir", help="Path for uncompressed taxdump.tar.gz", type=str)
    parser_a.add_argument("top_taxonomy_id", help="the top taxonomy want to search (taxon_id: 4751 for fungi)",
                          type=str)
    parser_a.add_argument("search_rank", help="rank want to search such as order", type=str)
    parser_a.add_argument("-o", "--output_file", help="Output file name (default as $STDOUT)", default=None, type=str)

    # argparse for CommonTree
    parser_a = subparsers.add_parser('CommonTree',
                                     help='get common tree by leaf taxon ID', description='')
    parser_a.add_argument("tax_dir", help="Path for uncompressed taxdump.tar.gz", type=str)
    parser_a.add_argument("tax_id_list", help="The file containing the tax_id to query", type=str)
    parser_a.add_argument("-o", "--output_file", help="Output file name (default as $STDOUT)", default=None, type=str)
    parser_a.add_argument("-id", "--ID_flag", help="output tree will writen by taxon id", action='store_true')
    parser_a.add_argument("-p", "--no_pass_node", help="remove pass node", action='store_true')

    # argparse for DiamondTaxonAssign
    parser_a = subparsers.add_parser('DiamondTaxonAssign',
                                     help='parse blast results to get protein seq taxon', description='')
    parser_a.add_argument("blast_results",
                          help="blast results need diamond output: qseqid sseqid staxids pident length mismatch gapopen qstart qend sstart send evalue bitscore",
                          type=str)
    parser_a.add_argument("tax_dir", help="Path for uncompressed taxdump.tar.gz", type=str)
    parser_a.add_argument("-n", "--num_threads", help="cpu number (default as 10)", default=10, type=int)
    parser_a.add_argument("-o", "--output_file", help="Output file name (default as taxon.assign.txt)",
                          default="taxon.assign.txt", type=str)

    # argparse for ExcludeTaxon
    parser_a = subparsers.add_parser('ExcludeTaxon',
                                     help='Exclude a taxon id, to get all taxon_id not include given taxon id',
                                     description='')
    parser_a.add_argument("taxon_id", help="taxon id which want to be excluded", type=str)
    parser_a.add_argument("tax_dir", help="Path for uncompressed taxdump.tar.gz", type=str)
    parser_a.add_argument("-r", "--root_taxon_id", help="root for excluded (default as root for all database)",
                          default='1', type=str)

    args = parser.parse_args()
    args_dict = vars(args)

    # --------------------------------------------
    #### command detail

    if args_dict["subcommand_name"] == "ID2Lineage":
        from lib.xuyuxing.evolution.taxonomy import build_taxon_database
        from lib.common.fileIO import read_list_file

        taxonomy_dir = args.tax_dir
        query_file = args.query_tax_id_file
        output_file = args.output_file

        # output as a tsv file

        tax_record_dict = build_taxon_database(taxonomy_dir)

        query_tax_id = read_list_file(query_file)
        if query_file is not None:
            query_tax_id = read_list_file(query_file)
        else:
            query_tax_id = tax_record_dict.keys()

        if output_file is not None:
            with open(output_file, 'w') as f:
                for tax_id in query_tax_id:
                    if tax_id in tax_record_dict:
                        taxon = tax_record_dict[tax_id]
                        lineage_str = ""
                        lineage_list = taxon.get_lineage(tax_record_dict)
                        for i in lineage_list:
                            tax_id_tmp, rank_tmp = i
                            lineage_str = lineage_str + tax_record_dict[tax_id_tmp].sci_name + " (" + rank_tmp + ");"
                        lineage_str = lineage_str.rstrip(";")
                        f.write(tax_id + "\t" + lineage_str + "\n")
                    else:
                        lineage_str = "error"
                        f.write(tax_id + "\t" + lineage_str + "\n")
        else:
            for tax_id in query_tax_id:
                taxon = tax_record_dict[tax_id]
                lineage_str = ""
                lineage_list = taxon.get_lineage(tax_record_dict)
                for i in lineage_list:
                    tax_id_tmp, rank_tmp = i
                    lineage_str = lineage_str + tax_record_dict[tax_id_tmp].sci_name + " (" + rank_tmp + ");"
                lineage_str = lineage_str.rstrip(";")
                print(tax_id + "\t" + lineage_str)

    elif args_dict["subcommand_name"] == "StoreTaxonDB":
        from lib.xuyuxing.evolution.taxonomy import build_taxon_database, store_tax_record_into_sqlite
        tax_record_dict = build_taxon_database(args.tax_dir)
        store_tax_record_into_sqlite(tax_record_dict, args.tax_db_file)


    # elif args_dict["subcommand_name"] == "CommonTree":
    #     from lib.evolution.taxonomy import build_taxon_database
    #     from lib.common.fileIO import read_list_file
    #     import ete3


    #     def get_common_tree(lineages, ID_flag):
    #         if ID_flag:
    #             tree = ete3.Tree(name='1')
    #         else:
    #             tree = ete3.Tree(name='root')

    #         for line_tmp in lineages:
    #             node_tmp = tree
    #             for i in line_tmp:
    #                 if ID_flag:
    #                     i = i[0]
    #                 else:
    #                     i = i[1]

    #                 if node_tmp.name == i:
    #                     continue
    #                 else:
    #                     child_flag = len([j for j in node_tmp.children if j.name == i])
    #                     if child_flag:
    #                         node_tmp = [j for j in node_tmp.children if j.name == i][0]
    #                     else:
    #                         node_tmp.add_child(name=i)
    #                         node_tmp = [j for j in node_tmp.children if j.name == i][0]
    #         return tree


    #     taxonomy_dir = args.tax_dir
    #     query_file = args.tax_id_list
    #     output_file = args.output_file
    #     id_flag = args.ID_flag

    #     query_list = read_list_file(query_file)
    #     tax_record_dict = build_taxon_database(taxonomy_dir)

    #     all_lineage = []
    #     for tax_id in query_list:
    #         taxon = tax_record_dict[tax_id]
    #         lineage_list = taxon.get_lineage(tax_record_dict)
    #         lineage_list_add = []
    #         for i in lineage_list:
    #             tax_id_tmp, rank_tmp = i
    #             lineage_list_add.append((tax_id_tmp, tax_record_dict[tax_id_tmp].sci_name, rank_tmp))
    #         all_lineage.append(lineage_list_add)

    #     tree = get_common_tree(all_lineage, id_flag)

    #     if output_file is None:
    #         print(tree)
    #     else:
    #         tree.write(format=1, outfile=output_file)


    elif args_dict["subcommand_name"] == "CommonTree":
        from lib.xuyuxing.evolution.taxonomy import build_taxon_database
        from lib.common.fileIO import read_list_file
        from lib.common.evolution.tree_operate import Tree, Clade, Phylo, draw_ascii, remove_pass_node

        def get_common_tree(lineages, ID_flag):
            if ID_flag:
                root_clade = Clade(name='1')
            else:
                root_clade = Clade(name='root')

            for line_tmp in lineages:
                node_tmp = root_clade
                for i in line_tmp:
                    if ID_flag:
                        i = i[0]
                    else:
                        i = i[1]

                    if node_tmp.name == i:
                        continue
                    else:
                        child_flag = len([j for j in node_tmp.clades if j.name == i])
                        if child_flag:
                            node_tmp = [j for j in node_tmp.clades if j.name == i][0]
                        else:
                            node_tmp.clades.append(Clade(name=i, branch_length=1.0))
                            node_tmp = [j for j in node_tmp.clades if j.name == i][0]

            tree = Tree(root_clade, True)
            return tree

        taxonomy_dir = args.tax_dir
        query_file = args.tax_id_list
        output_file = args.output_file
        id_flag = args.ID_flag
        no_pass_node = args.no_pass_node

        query_list = read_list_file(query_file)
        tax_record_dict = build_taxon_database(taxonomy_dir)

        all_lineage = []
        for tax_id in query_list:
            taxon = tax_record_dict[tax_id]
            lineage_list = taxon.get_lineage(tax_record_dict)
            lineage_list_add = []
            for i in lineage_list:
                tax_id_tmp, rank_tmp = i
                lineage_list_add.append((tax_id_tmp, tax_record_dict[tax_id_tmp].sci_name, rank_tmp))
            all_lineage.append(lineage_list_add)

        tree = get_common_tree(all_lineage, id_flag)

        if no_pass_node:
            tree = remove_pass_node(tree)

        if output_file is None:
            draw_ascii(tree)
        else:
            with open(output_file, 'w') as f:
                Phylo.write(tree, f, 'newick')


    elif args_dict["subcommand_name"] == "Name2Lineage":
        from lib.xuyuxing.evolution.taxonomy import build_taxon_database
        from lib.common.fileIO import read_list_file
        from fuzzywuzzy import process
        import sys


        taxonomy_dir = args.tax_dir
        query_file = args.query_file
        output_file = args.output_file
        fuzzy_search = args.fuzzy_search

        # build database
        tax_record_dict = build_taxon_database(taxonomy_dir, True)

        # make hash table
        tax_record_sciname_hash = {}
        tax_record_othername_hash = {}
        for tax_id in tax_record_dict:
            tax_tmp = tax_record_dict[tax_id]

            if tax_tmp.sci_name not in tax_record_sciname_hash:
                tax_record_sciname_hash[tax_tmp.sci_name] = []
            tax_record_sciname_hash[tax_tmp.sci_name].append(tax_id)

            if hasattr(tax_tmp, 'other_name'):
                for name_tmp in tax_tmp.other_name:
                    if name_tmp not in tax_record_othername_hash:
                        tax_record_othername_hash[name_tmp] = []
                    tax_record_othername_hash[name_tmp].append(tax_id)

        # read query file
        query_tax_name = read_list_file(query_file)

        # search query in hash table
        subject_list = []
        sci_name_list = list(tax_record_sciname_hash.keys())
        other_name_list = list(tax_record_othername_hash.keys())
        for search_string in query_tax_name:
            other_name_flag = False
            fuzzy_flag = False
            homonym_flag = False
            # search sci_name
            if search_string in sci_name_list:
                search_out = tax_record_sciname_hash[search_string]
            # search other_name
            elif search_string in other_name_list:
                other_name_flag = True
                search_out = tax_record_othername_hash[search_string]
            # fuzzy search all name
            elif fuzzy_search:
                fuzzy_flag = True
                search_out = tax_record_sciname_hash[
                    process.extractOne(search_string, sci_name_list + other_name_list)[0]]
            else:
                search_out = []
            # remove older taxid
            search_out = list(set([tax_record_dict[i].tax_id for i in search_out]))

            # check if homonym
            if len(search_out) > 1:
                homonym_flag = True
                print(search_out)
            # add failed info for None
            if len(search_out) == 0:
                subject_list.append((search_string,))
            # add to subject_list
            for tax_id in search_out:
                tax_tmp = tax_record_dict[tax_id]
                subject_list.append((search_string, tax_tmp.tax_id, tax_tmp.sci_name,
                                     tax_tmp.get_lineage(tax_record_dict), other_name_flag, fuzzy_flag, homonym_flag))

        if output_file is None:
            file = sys.stdout
        else:
            file = open(output_file, 'w')

        file.write("query\ttax_id\tsci_name\tlineage\tother_name_flag\tfuzzy_flag\thomonym_flag\n")
        for i in subject_list:
            if len(i) == 1:
                file.write(i[0] + "\terror\n")
            else:
                lineage_str = ""
                lineage_list = i[3]
                for j in lineage_list:
                    tax_id_tmp, rank_tmp = j
                    lineage_str = lineage_str + tax_record_dict[tax_id_tmp].sci_name + " (" + rank_tmp + ");"
                lineage_str = lineage_str.rstrip(";")
                file.write("%s\t%s\t%s\t%s\t%s\t%s\t%s\n" % (i[0], i[1], i[2], lineage_str, i[4], i[5], i[6]))

    elif args_dict["subcommand_name"] == "RankStat":
        from lib.xuyuxing.evolution.taxonomy import build_taxon_database
        from lib.common.fileIO import read_list_file
        import sys


        # build database
        tax_record_dict = build_taxon_database(args.tax_dir, True)

        # get list
        want_id_list = []
        for tax_id in tax_record_dict:
            tax_tmp = tax_record_dict[tax_id]
            if tax_tmp.rank == args.search_rank and tax_tmp.is_child_of(args.top_taxonomy_id, tax_record_dict):
                want_id_list.append(tax_id)

        # output
        if not args.output_file is None:
            sys.stdout = open(args.output_file, 'w')

        print("tax_id\tsci_name\trank\tlineage")
        for tax_id in want_id_list:
            tax_tmp = tax_record_dict[tax_id]
            lineage_str = ""
            lineage_list = tax_tmp.get_lineage(tax_record_dict)
            for j in lineage_list:
                tax_id_tmp, rank_tmp = j
                lineage_str = lineage_str + tax_record_dict[tax_id_tmp].sci_name + " (" + rank_tmp + ");"
            lineage_str = lineage_str.rstrip(";")

            print("%s\t%s\t%s\t%s" % (tax_id, tax_tmp.sci_name, tax_tmp.rank, lineage_str))

    elif args_dict["subcommand_name"] == "ExcludeTaxon":
        """
        class abc():
            pass
        
        args = abc()
        args.tax_dir = '/lustre/home/xuyuxing/Database/genome2020/genome/info/NCBI/taxonomy'
        args.root_taxon_id = '73496'
        args.taxon_id = '40553'
        """

        from lib.xuyuxing.evolution.taxonomy import build_taxon_database
        from lib.common.fileIO import read_list_file

        import copy
        from lib.common.util import printer_list

        tax_record_dict = build_taxon_database(args.tax_dir)
        root_taxon = tax_record_dict[args.root_taxon_id]
        exc_taxon = tax_record_dict[args.taxon_id]

        # get_lineage
        root_lineage = root_taxon.get_lineage(tax_record_dict)
        exc_lineage = exc_taxon.get_lineage(tax_record_dict)

        # diff lineage
        diff_lineage = copy.deepcopy(exc_lineage)
        for i in root_lineage[:-1]:
            diff_lineage.remove(i)

        output_list = []
        for i in range(len(diff_lineage) - 1):
            tax_id, rank_note = diff_lineage[i]
            next_tax_id, rank_note = diff_lineage[i + 1]

            tax_tmp = tax_record_dict[tax_id]
            output_list.extend(list(set([j for j in tax_tmp.get_sons(tax_record_dict) if j != next_tax_id])))

        print(printer_list(output_list, ','))

    elif args_dict["subcommand_name"] == "DiamondTaxonAssign":
        """
        class abc():
            pass

        args = abc()
        args.blast_results = '/lustre/home/xuyuxing/Database/Cuscuta/Cau/genomev1.1/gene_taxon/Cuscuta.pt.v1.1.fasta.bls'
        args.tax_dir = '/lustre/home/xuyuxing/Database/NCBI/nr/2020/taxdmp'
        args.output_file = '/lustre/home/xuyuxing/Database/Cuscuta/Cau/genomev1.1/gene_taxon/Cuscuta.pt.v1.1.fasta.taxon'
        args.num_threads = 56
        """
        from src.xuyuxing.tools.taxontools import DiamondTaxonAssign_main
        DiamondTaxonAssign_main(args)



import numpy as np
from matplotlib import cm, rc, rcParams
import matplotlib.pyplot as plt
import matplotlib as mpl
from matplotlib.lines import Line2D
from matplotlib.path import Path
import matplotlib.patches as Patches

from lib.xuyuxing.base.console import green, dark
import copy
import os.path as op
import sys
import logging
import matplotlib.cbook as cbook


def save_figure(fig_object, output_file, format='svg'):
    fig_object.savefig(output_file, format=format,
                       facecolor='none', edgecolor='none', bbox_inches='tight')


# def savefig(figname, dpi=150, iopts=None, cleanup=True):
#     try:
#         format = figname.rsplit(".", 1)[-1].lower()
#     except:
#         format = "pdf"
#     try:
#         plt.savefig(figname, dpi=dpi, format=format)
#     except Exception as e:
#         message = "savefig failed. Reset usetex to False."
#         message += "\n{0}".format(str(e))
#         rc("text", usetex=False)
#         plt.savefig(figname, dpi=dpi)

#     msg = "Figure saved to `{0}`".format(figname)
#     if iopts:
#         msg += " {0}".format(iopts)

#     if cleanup:
#         plt.rcdefaults()


def load_image(input_file):
    with cbook.get_sample_data(input_file) as image_file:
        image = plt.imread(image_file)
    return image


def quick_show_image_from_file(input_file):

    fig, ax = plt.subplots()

    im = load_image(input_file)

    ax.imshow(im)
    ax.axis('off')

    plt.show()


# def load_image(filename):
#     img = plt.imread(filename)
#     if len(img.shape) == 2:  # Gray-scale image, convert to RGB
#         # http://www.socouldanyone.com/2013/03/converting-grayscale-to-rgb-with-numpy.html
#         h, w = img.shape
#         ret = np.empty((h, w, 3), dtype=np.uint8)
#         ret[:, :, 2] = ret[:, :, 1] = ret[:, :, 0] = img
#         img = ret
#     else:
#         h, w, c = img.shape
#     print("Image `{0}` loaded ({1}px x {2}px).".format(filename, w, h))
#     return img

# ascii


def asciiaxis(x, digit=1):
    if isinstance(x, int):
        x = str(x)
    elif isinstance(x, float):
        x = "{0:.{1}f}".format(x, digit)
    elif isinstance(x, np.int64):
        x = str(x)
    elif isinstance(x, np.ndarray):
        assert len(x) == 2
        x = str(x).replace("]", ")")  # upper bound not inclusive

    return x


def asciiplot(x, y, digit=1, width=50, title=None, char="="):
    """
    Print out a horizontal plot using ASCII chars.
    width is the textwidth (height) of the plot.
    """
    ax = np.array(x)
    ay = np.array(y)

    if title:
        print(dark(title), file=sys.stderr)

    az = ay * width // ay.max()
    tx = [asciiaxis(x, digit=digit) for x in ax]
    rjust = max([len(x) for x in tx]) + 1

    for x, y, z in zip(tx, ay, az):
        x = x.rjust(rjust)
        y = y or ""
        z = green(char * z)
        print("{0} |{1} {2}".format(x, z, y), file=sys.stderr)


# litter plot
def add_stick_box_plot(start, end, high=5, x_y_lim=None, thickness=2, linewidth=2, alpha=0.5, facecolor='b', edgecolor='k', bulge=2):
    if not x_y_lim:
        circle_radius = thickness/bulge
    else:
        x_lim, y_lim = x_y_lim
        x_y_ratio = (max(x_lim) - min(x_lim)) / (max(y_lim) - min(y_lim))
        circle_radius = (thickness/bulge) * x_y_ratio

    if end - start > 2 * circle_radius:

        start = start + circle_radius
        end = end - circle_radius

        path_data = [
            # up line
            (Path.MOVETO, [start, high+thickness/2]),
            (Path.LINETO, [end, high+thickness/2]),
            # right CURVE4
            (Path.CURVE4, [end+circle_radius, high+thickness/2]),
            (Path.CURVE4, [end+circle_radius, high-thickness/2]),
            (Path.CURVE4, [end, high-thickness/2]),
            # down line
            (Path.LINETO, [start, high-thickness/2]),
            # left CURVE4
            (Path.CURVE4, [start-circle_radius, high-thickness/2]),
            (Path.CURVE4, [start-circle_radius, high+thickness/2]),
            (Path.CURVE4, [start, high+thickness/2]),
        ]

        codes, verts = zip(*path_data)
        path = Path(verts, codes)

        patch = Patches.PathPatch(
            path, alpha=alpha, facecolor=facecolor, edgecolor=edgecolor, linewidth=linewidth)

    else:
        patch = Patches.Circle(((end + start)/2, high), radius=circle_radius,
                               alpha=alpha, facecolor=facecolor, edgecolor=edgecolor, linewidth=linewidth)

    return patch


def add_small_axes_on_other(main_ax, figure, left_bottom_xy, right_top_xy):
    """
    add a new axes on a main axes by left_bottom_xy and right_top_xy

    left_bottom_xy = (2,2)
    right_top_xy = (3,4)
    """

    # print(ax.transData.transform((0, 4)))
    lb_xy = figure.transFigure.inverted().transform(
        main_ax.transData.transform(left_bottom_xy))
    rt_xy = figure.transFigure.inverted().transform(
        main_ax.transData.transform(right_top_xy))

    return figure.add_axes([lb_xy[0], lb_xy[1], rt_xy[0]-lb_xy[0], rt_xy[1]-lb_xy[1]])



from lib.common.math.bin_and_window import split_sequence_to_bins
from pyfaidx import Fasta
import pysam
import io
import sys


def ContigDepth_main(args):

    ref_genome = Fasta(args.ref_genome)

    depth_sum = {}
    cover_site = {}
    for contig in ref_genome.keys():
        depth_sum[contig] = 0
        cover_site[contig] = 0
        contig_length = len(ref_genome[contig])
        for i, start, end in split_sequence_to_bins(contig_length, 10000, start=1):
            fancy_name = "%s:%d-%d" % (contig, start, end)
            depth_string = pysam.depth(
                "-a", "-r", fancy_name, args.bam_file)
            for each_line in io.StringIO(depth_string):
                cover_reads = int(each_line.split()[2])
                depth_sum[contig] = depth_sum[contig] + cover_reads
                if cover_reads > 0:
                    cover_site[contig] = cover_site[contig] + 1

    with open(args.output_file, 'w') as f:
        f.write("contig\tlength\tdepth_sum\tavg_depth\tcover_base\tcoverage\n")
        for contig in ref_genome.keys():
            f.write("%s\t%d\t%d\t%.2f\t%d\t%.2f%%\n" % (
                contig, len(
                    ref_genome[contig]), depth_sum[contig], depth_sum[contig] / len(ref_genome[contig]),
                cover_site[contig], cover_site[contig] * 100 / len(ref_genome[contig])))
